# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dbmail', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mailfromemail',
            name='email',
            field=models.CharField(help_text='For sms/tts you must specify name or number', unique=True, max_length=75, verbose_name='Email'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='mailgroupemail',
            name='email',
            field=models.CharField(help_text=b'For sms/tts you must specify number', max_length=75, verbose_name='Email'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='maillogemail',
            name='email',
            field=models.CharField(max_length=75),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='mailtemplate',
            name='context_note',
            field=models.TextField(help_text='This is simple note field for context variables with description.', null=True, verbose_name='Context note', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='mailtemplate',
            name='from_email',
            field=models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, default=None, to='dbmail.MailFromEmail', blank=True, help_text='If not specified, then used default.', null=True, verbose_name='Message from'),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='mailtemplate',
            name='is_html',
            field=models.BooleanField(default=True, help_text='For sms/tts must be text not html', verbose_name='Is html'),
            preserve_default=True,
        ),
    ]



# -*- encoding: utf-8 -*-

from dbmail.defaults import DEFAULT_PUSH_FROM, PUSH_PROVIDER
from dbmail.backends.sms import Sender as SenderBase
from dbmail.utils import clean_html
from dbmail import import_module


class Sender(SenderBase):
    provider = PUSH_PROVIDER
    default_from = DEFAULT_PUSH_FROM

    def _get_recipient_list(self, recipient):
        if not isinstance(recipient, list):
            email_list = self._group_emails(recipient)
            if email_list:
                return email_list
        return self._email_to_list(recipient)

    def _send(self):
        message = clean_html(self._message)

        options = self._kwargs.copy()
        options.update(
            message=message,
            event=self._kwargs.pop('event', self._subject)
        )
        if self._from_email:
            options['app'] = self._from_email

        module = import_module(self._provider)

        for address in self._recipient_list:
            module.send(address, **options)


class SenderDebug(Sender):
    def _send(self):
        self.debug('Provider', self._provider or self.provider)
        self.debug('Message', clean_html(self._message))
        self.debug('Recipients', self._recipient_list)
        self.debug('Event', self._subject)
        self.debug('App', self._from_email)



from dbmail.providers.microsoft.base import MPNSBase, ElementTree


class MPNSToast(MPNSBase):
    NOTIFICATION_CLS = 2
    TARGET = 'toast'

    def payload(self, payload):
        root = ElementTree.Element("{WPNotification}Notification")
        toast = ElementTree.SubElement(root, '{WPNotification}Toast')
        self.sub(toast, '{WPNotification}Text1', 'text1', payload)
        self.sub(toast, '{WPNotification}Text2', 'text2', payload)
        self.sub(toast, '{WPNotification}Sound', 'sound', payload)
        self.sub(toast, '{WPNotification}Param', 'param', payload)
        self.sub(toast, '{WPNotification}Path', 'path', payload)
        return self.serialize_tree(ElementTree.ElementTree(root))


def send(uri, message, **kwargs):
    kwargs['text1'] = kwargs.pop("event", 'App')
    kwargs['text2'] = message
    return MPNSToast().send(uri, kwargs)



# -*- encoding: utf-8 -*-

try:
    from httplib import HTTPSConnection
    from urllib import urlencode
except ImportError:
    from http.client import HTTPSConnection
    from urllib.parse import urlencode

from django.conf import settings
from dbmail import get_version


class ProwlError(Exception):
    pass


def from_unicode(text, text_length=None):
    try:
        text = text.encode('utf-8', 'ignore')
    except UnicodeDecodeError:
        pass

    if text_length is not None:
        text = text[0:text_length]

    return text


def send(api_key, description, **kwargs):
    """
    Site: http://prowlapp.com
    API: http://prowlapp.com/api.php
    Desc: Best app for system administrators
    """
    headers = {
        "User-Agent": "DBMail/%s" % get_version(),
        "Content-type": "application/x-www-form-urlencoded"
    }

    application = from_unicode(kwargs.pop("app", settings.PROWL_APP), 256)
    event = from_unicode(kwargs.pop("event", 'Alert'), 1024)
    description = from_unicode(description, 10000)

    data = {
        "apikey": api_key,
        "application": application,
        "event": event,
        "description": description,
        "priority": kwargs.pop("priority", 1)
    }

    provider_key = kwargs.pop("providerkey", None)
    url = kwargs.pop('url', None)

    if provider_key is not None:
        data["providerkey"] = provider_key

    if url is not None:
        data["url"] = url[0:512]

    http = HTTPSConnection(kwargs.pop("api_url", "api.prowlapp.com"))
    http.request(
        "POST", "/publicapi/add",
        headers=headers,
        body=urlencode(data))
    response = http.getresponse()

    if response.status != 200:
        raise ProwlError(response.reason)
    return True



# -*- encoding: utf-8 -*-

import traceback
import pprint
import uuid
import time
import sys

from django.db.models.fields.related import ManyToManyField, ForeignKey
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.core.urlresolvers import reverse, NoReverseMatch
from django.contrib.sites.models import Site
from django.template import Template, Context
from django.core.mail import get_connection
from django.utils import translation
from django.conf import settings
from django.core import signing

from dbmail.models import MailTemplate, MailLog, MailGroup, MailLogException
from dbmail.defaults import SHOW_CONTEXT, ENABLE_LOGGING, ADD_HEADER
from dbmail.exceptions import StopSendingException
from dbmail.utils import clean_html
from dbmail import import_module
from dbmail import get_version
from dbmail import defaults


class Sender(object):
    provider = defaults.MAIL_PROVIDER

    def __init__(self, slug, recipient, *args, **kwargs):
        self._slug = slug

        self._recipient_list = self._get_recipient_list(recipient)
        self._cc = self._email_to_list(kwargs.pop('cc', None))
        self._bcc = self._email_to_list(kwargs.pop('bcc', None))
        self._user = kwargs.pop('user', None)
        self._language = kwargs.pop('language', None)
        self._backend = kwargs.pop('backend')
        self._provider = kwargs.pop('provider', self.provider)
        self._signals_kw = kwargs.pop('signals_kwargs', {})

        self._template = self._get_template()
        self._context = self._get_context(args)

        self._subject = self._get_subject()
        self._message = self._get_message()
        self._files = kwargs.pop('files', [])
        self._kwargs = kwargs
        self._num = 1
        self._err_msg = None
        self._err_exc = None
        self._log_id = self._get_log_id()

        self._kwargs.pop('retry', None)
        self._kwargs.pop('max_retries', None)
        self._kwargs.pop('retry_delay', None)

        self._from_email = self._get_from_email()
        self._update_bcc_from_template_settings()
        self._insert_mailer_identification_head()

    @staticmethod
    def _get_log_id():
        return '%f-%s' % (time.time(), uuid.uuid4())

    def _insert_mailer_identification_head(self):
        if not ADD_HEADER:
            return
        headers = self._kwargs.pop('headers', {})
        headers.update(
            {'X-Mailer-Wrapper': 'django-db-mailer ver %s' % get_version()})
        self._kwargs['headers'] = headers

    def _get_connection(self):
        if self._template.auth_credentials:
            return self._kwargs.pop('connection', None) or get_connection(
                **self._template.auth_credentials)
        return self._kwargs.pop('connection', None)

    def _get_template(self):
        return MailTemplate.get_template(slug=self._slug)

    def _get_context(self, context_list):
        try:
            data = self._model_to_dict(Site.objects.get_current())
        except Site.DoesNotExist:
            data = {}

        for context in context_list:
            if isinstance(context, dict):
                data.update(context)
            elif hasattr(context, '_meta'):
                data.update(self._model_to_dict(context))
                data.update({self._get_context_module_name(context): context})

        if settings.DEBUG and SHOW_CONTEXT:
            pprint.pprint(data)
        return data

    @staticmethod
    def _get_context_module_name(context):
        from distutils.version import StrictVersion
        import django

        current_version = django.get_version()

        if StrictVersion(current_version) < StrictVersion('1.8'):
            return context._meta.module_name
        return context._meta.model_name

    def _get_str_by_language(self, field, template=None):
        obj = template if template else self._template
        template = getattr(obj, field)
        if self._language is not None:
            field = '%s_%s' % (field, self._language)
            if hasattr(obj, field):
                if getattr(obj, field):
                    template = getattr(obj, field)
        return template

    def _get_subject(self):
        return self._render_template(
            self._get_str_by_language('subject'), self._context)

    def _get_message_with_base(self):
        self._context['content'] = self._render_template(
            self._get_str_by_language('message'), self._context)
        return self._render_template(
            self._get_str_by_language('message', self._template.base),
            self._context
        )

    def _get_standard_message(self):
        return self._render_template(
            self._get_str_by_language('message'), self._context)

    def _get_message(self):
        if self._template.base:
            return self._get_message_with_base()
        return self._get_standard_message()

    def _get_msg_with_track(self):
        message = self._message
        if defaults.TRACK_ENABLE is False:
            return message
        if ENABLE_LOGGING and self._template.enable_log:
            try:
                domain = Site.objects.get_current().domain
                encrypted = signing.dumps(self._log_id, compress=True)
                path = reverse('db-mail-tracker', args=[encrypted])
                message += defaults.TRACK_HTML % {
                    'url': 'http://%s%s' % (domain, path)}
            except (Site.DoesNotExist, NoReverseMatch):
                pass
        return message

    def _attach_files(self, mail):
        for file_object in self._template.files_list:
            mail.attach_file(file_object.filename.path)

        for filename in self._files:
            mail.attach_file(filename)

    def _send_html_message(self):
        msg = EmailMultiAlternatives(
            self._subject, clean_html(self._message), cc=self._cc,
            from_email=self._from_email, to=self._recipient_list,
            bcc=self._bcc, connection=self._get_connection(), **self._kwargs
        )
        msg.attach_alternative(self._get_msg_with_track(), "text/html")
        self._attach_files(msg)
        msg.send()

    def _send_plain_message(self):
        msg = EmailMessage(
            self._subject, self._message, from_email=self._from_email,
            to=self._recipient_list, cc=self._cc, bcc=self._bcc,
            connection=self._get_connection(), **self._kwargs
        )
        self._attach_files(msg)
        msg.send()

    def _get_recipient_list(self, recipient):
        if not isinstance(recipient, list) and '@' not in recipient:
            return self._group_emails(recipient)
        return self._email_to_list(recipient)

    def _update_bcc_from_template_settings(self):
        if self._template.bcc_list:
            if self._bcc:
                self._bcc.extend(self._template.bcc_list)
            else:
                self._bcc = self._template.bcc_list

    def _get_from_email(self):
        if self._kwargs.get('from_email'):
            return self._kwargs.pop('from_email', None)
        elif not self._template.from_email:
            return settings.DEFAULT_FROM_EMAIL
        return self._template.from_email.get_mail_from

    @staticmethod
    def _group_emails(recipient):
        email_list = []
        for slug in recipient.split(','):
            email_list.extend(MailGroup.get_emails(slug))
        return list(set(email_list))

    @staticmethod
    def _email_to_list(recipient):
        if recipient is None:
            return None
        elif not isinstance(recipient, list):
            recipient = [d.strip() for d in recipient.split(',') if d.strip()]
        return recipient

    def _render_template(self, template, context):
        translation.activate(self._language or settings.LANGUAGE_CODE)
        return Template(template).render(Context(context))

    @staticmethod
    def _model_to_dict(instance):
        opts, data = getattr(instance, '_meta'), dict()
        for f in opts.fields + opts.many_to_many:
            if isinstance(f, ManyToManyField):
                if instance.pk is None:
                    data[f.name] = []
                else:
                    data[f.name] = list(f.value_from_object(
                        instance).values_list('pk', flat=True))
            elif isinstance(f, ForeignKey):
                if getattr(instance, f.name):
                    data[f.name] = getattr(instance, f.name).__unicode__()
            else:
                data[f.name] = f.value_from_object(instance)
        return data

    def _send_by_custom_provider(self):
        module = import_module(self._provider)
        module.send(self)

    def _send_by_native_provider(self):
        if self._template.is_html:
            return self._send_html_message()
        return self._send_plain_message()

    def _send(self):
        if self._provider is not None:
            return self._send_by_custom_provider()
        return self._send_by_native_provider()

    def _store_log(self, is_sent):
        if ENABLE_LOGGING is True:
            if self._template.enable_log or not is_sent:
                MailLog.store(
                    self._recipient_list, self._cc, self._bcc,
                    is_sent, self._template, self._user,
                    self._num, self._err_msg, self._err_exc,
                    self._log_id, self._backend, self._provider
                )

    def _try_to_send(self):
        self._kwargs.pop('queue', None)
        for self._num in range(1, self._template.num_of_retries + 1):
            try:
                self._send()
                break
            except Exception as exc:
                print('[dbmail] %s' % exc)
                if self._template.num_of_retries == self._num:
                    raise
                time.sleep(defaults.SEND_RETRY_DELAY_DIRECT)

    def _ignore_exception(self):
        return self._err_exc in MailLogException.get_ignored_exceptions()

    def send(self, is_celery=True):
        from dbmail.signals import pre_send, post_send, post_exception

        if self._template.is_active:
            try:
                pre_send.send(
                    self.__class__, instance=self, **self._signals_kw)
                if is_celery is True:
                    self._send()
                else:
                    self._try_to_send()
                self._store_log(True)
                post_send.send(
                    self.__class__, instance=self, **self._signals_kw)
                return 'OK'
            except StopSendingException:
                return
            except Exception as exc:
                post_exception.send(
                    self.__class__,
                    instance=self,
                    exc_instance=exc,
                    **self._signals_kw
                )
                self._err_msg = traceback.format_exc()
                self._err_exc = exc.__class__.__name__
                self._store_log(False)
                if self._ignore_exception():
                    return
                raise

    @staticmethod
    def debug(key, value):
        from django.utils.termcolors import colorize

        if value:
            sys.stdout.write(colorize(key, fg='green'))
            sys.stdout.write(": ")
            sys.stdout.write(colorize(repr(value), fg='white'))
            sys.stdout.write("\n")


class SenderDebug(Sender):
    def _send(self):
        self.debug('Provider', self._provider or 'default')
        self.debug('Message', self._message)
        self.debug('From', self._from_email)
        self.debug('Recipients', self._recipient_list)
        self.debug('CC', self._cc)
        self.debug('BCC', self._bcc)
        self.debug('Additional kwargs', self._kwargs)



# -*- encoding: utf-8 -*-

from binascii import a2b_hex
from json import dumps
from socket import socket, AF_INET, SOCK_STREAM
from struct import pack
from time import time

try:
    from ssl import wrap_socket
except ImportError:
    from socket import ssl as wrap_socket

from django.conf import settings

from dbmail import defaults
from dbmail.providers.apple.errors import APNsError
from dbmail import PY3


def send(token_hex, message, **kwargs):
    """
    Site: https://apple.com
    API: https://developer.apple.com
    Desc: iOS notifications
    """
    is_enhanced = kwargs.pop('is_enhanced', False)
    identifier = kwargs.pop('identifier', 0)
    expiry = kwargs.pop('expiry', 0)

    alert = {
        "title": kwargs.pop("event"),
        "body": message,
        "action": kwargs.pop(
            'apns_action', defaults.APNS_PROVIDER_DEFAULT_ACTION)
    }

    data = {
        "aps": {
            'alert': alert,
            'content-available': kwargs.pop('content_available', 0) and 1
        }
    }
    data['aps'].update(kwargs)
    payload = dumps(data, separators=(',', ':'))

    token = a2b_hex(token_hex)
    if is_enhanced is True:
        fmt = '!BIIH32sH%ds' % len(payload)
        expiry = expiry and time() + expiry
        notification = pack(
            fmt, 1, identifier, expiry,
            32, token, len(payload), payload)
    else:
        token_length_bin = pack('>H', len(token))
        payload_length_bin = pack('>H', len(payload))
        zero_byte = bytes('\0', 'utf-8') if PY3 is True else '\0'
        notification = (
            zero_byte + token_length_bin + token +
            payload_length_bin + payload)

    sock = socket(AF_INET, SOCK_STREAM)
    sock.settimeout(3)
    sock.connect((settings.APNS_GW_HOST, settings.APNS_GW_PORT))
    ssl = wrap_socket(
        sock, settings.APNS_KEY_FILE,
        settings.APNS_CERT_FILE,
        do_handshake_on_connect=False)

    result = ssl.write(notification)

    sock.close()
    ssl.close()

    if not result:
        raise APNsError

    return True



# -*- encoding: utf-8 -*-

from datetime import datetime
import sys


VERSION = (2, 3, 14)

default_app_config = 'dbmail.apps.DBMailConfig'


def get_version():
    return '.'.join(map(str, VERSION))


def app_installed(app):
    from django.conf import settings

    return app in settings.INSTALLED_APPS


def celery_supported():
    try:
        from dbmail import tasks

        if not app_installed('djcelery'):
            raise ImportError
        return True
    except ImportError:
        return False


def db_sender(slug, recipient, *args, **kwargs):
    from dbmail.defaults import (
        CELERY_QUEUE, SEND_MAX_TIME, ENABLE_CELERY, BACKEND, DEBUG)
    from dbmail.models import MailTemplate

    args = (slug, recipient) + args
    send_after = kwargs.pop('send_after', None)
    send_at_date = kwargs.pop('send_at_date', None)
    _use_celery = kwargs.pop('use_celery', ENABLE_CELERY)
    use_celery = ENABLE_CELERY and _use_celery
    backend = kwargs.get('backend', BACKEND['mail'])

    if celery_supported() and use_celery is True:
        import dbmail.tasks

        template = MailTemplate.get_template(slug=slug)
        max_retries = kwargs.get('max_retries', None)
        send_after = send_after if send_after else template.interval
        if max_retries is None and template.num_of_retries:
            kwargs['max_retries'] = template.num_of_retries

        options = {
            'args': args, 'kwargs': kwargs,
            'queue': kwargs.pop('queue', CELERY_QUEUE),
            'time_limit': kwargs.get('time_limit', SEND_MAX_TIME),
            'priority': template.priority,
        }

        if send_at_date is not None and isinstance(send_at_date, datetime):
            options.update({'eta': send_at_date})
        if send_after is not None:
            options.update({'countdown': send_after})
        if template.is_active:
            return dbmail.tasks.db_sender.apply_async(**options)
    else:
        module = import_module(backend)
        if DEBUG is True:
            return module.SenderDebug(*args, **kwargs).send(is_celery=False)
        return module.Sender(*args, **kwargs).send(is_celery=False)


def send_db_mail(*args, **kwargs):
    from dbmail.defaults import BACKEND

    kwargs['backend'] = kwargs.pop('backend', BACKEND['mail'])
    return db_sender(*args, **kwargs)


def send_db_sms(*args, **kwargs):
    from dbmail.defaults import BACKEND, SMS_QUEUE

    kwargs['backend'] = kwargs.pop('backend', BACKEND['sms'])
    kwargs['queue'] = kwargs.pop('queue', SMS_QUEUE)
    return db_sender(*args, **kwargs)


def send_db_tts(*args, **kwargs):
    from dbmail.defaults import BACKEND, TTS_QUEUE

    kwargs['backend'] = kwargs.pop('backend', BACKEND['tts'])
    kwargs['queue'] = kwargs.pop('queue', TTS_QUEUE)
    return db_sender(*args, **kwargs)


def send_db_push(*args, **kwargs):
    from dbmail.defaults import BACKEND, PUSH_QUEUE

    kwargs['backend'] = kwargs.pop('backend', BACKEND['push'])
    kwargs['queue'] = kwargs.pop('queue', PUSH_QUEUE)
    return db_sender(*args, **kwargs)


def send_db_bot(*args, **kwargs):
    from dbmail.defaults import BACKEND, BOT_QUEUE

    kwargs['backend'] = kwargs.pop('backend', BACKEND['bot'])
    kwargs['queue'] = kwargs.pop('queue', BOT_QUEUE)
    return db_sender(*args, **kwargs)


def send_db_subscription(*args, **kwargs):
    from dbmail.defaults import (
        SUBSCRIPTION_QUEUE, SEND_MAX_TIME,
        ENABLE_CELERY, MAIL_SUBSCRIPTION_MODEL)

    MailSubscription = import_by_string(MAIL_SUBSCRIPTION_MODEL)

    use_celery = ENABLE_CELERY and kwargs.pop('use_celery', ENABLE_CELERY)
    options = {
        'time_limit': kwargs.pop('time_limit', SEND_MAX_TIME),
        'queue': kwargs.pop('queue', SUBSCRIPTION_QUEUE),
        'args': args, 'kwargs': kwargs,
    }

    if celery_supported() and use_celery is True:
        from dbmail.tasks import db_subscription

        return db_subscription.apply_async(**options)
    else:
        kwargs['use_celery'] = use_celery
        return MailSubscription.notify(*args, **kwargs)


def initial_signals():
    from django.db.utils import DatabaseError, IntegrityError

    for cmd in ['schemamigration', 'migrate', 'syncdb',
                'test', 'createsuperuser', 'makemigrations',
                'collectstatic', 'compilemessages']:
        if cmd in sys.argv:
            break
    else:
        try:
            from dbmail.signals import initial_signals as init_signals

            init_signals()
        except (ImportError, DatabaseError, IntegrityError):
            pass


##
# Compatibility section
##

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3


def python_2_unicode_compatible(klass):
    """
    A decorator that defines __unicode__ and __str__ methods under Python 2.
    Under Python 3 it does nothing.
    To support Python 2 and 3 with a single code base, define a __str__ method
    returning text and apply this decorator to the class.
    """
    if PY2:
        if '__str__' not in klass.__dict__:
            raise ValueError("@python_2_unicode_compatible cannot be applied "
                             "to %s because it doesn't define __str__()." %
                             klass.__name__)
        klass.__unicode__ = klass.__str__
        klass.__str__ = lambda self: self.__unicode__().encode('utf-8')
    return klass


def import_by_string(dotted_path):
    """Import class by his full module path.

    Function for compability of different Django versions.

    Args:
        dotted_path - string, full import path for class.

    """

    import_string = None

    # Django >= 1.5, < 1.7
    try:
        from django.utils.module_loading import import_by_path as import_string
    except ImportError:
        import_string = import_string

    # Django >= 1.7
    try:
        from django.utils.module_loading import import_string
    except ImportError:
        import_string = import_string

    if import_string is not None:
        return import_string(dotted_path)

    # Django == 1.4
    from django.utils.importlib import import_module

    class_data = dotted_path.split('.')
    module_path = '.'.join(class_data[:-1])
    class_str = class_data[-1]

    module = import_module(module_path)
    # Finally, we retrieve the Class
    return getattr(module, class_str)


def import_module(*args, **kwargs):
    try:
        from django.utils.importlib import import_module
    except ImportError:
        from importlib import import_module
    return import_module(*args, **kwargs)


def get_model(*args, **kwargs):
    try:
        from django.db.models import get_model as _get_model
    except ImportError:
        # Django > 1.8
        from django.apps import apps

        def _get_model(*args, **kwargs):
            return apps.get_model(*args, **kwargs)
    return _get_model(*args, **kwargs)



# -*- coding: utf-8 -*-

try:
    from httplib import HTTPConnection
    from urllib import urlencode
except ImportError:
    from http.client import HTTPConnection
    from urllib.parse import urlencode


from django.conf import settings

from dbmail.providers.prowl.push import from_unicode
from dbmail import get_version

import json


class AeroSmsError(Exception):
    pass


def send(sms_to, sms_body, **kwargs):
    """
    Site: http://smsaero.ru/
    API: http://smsaero.ru/api/
    """
    headers = {
        "User-Agent": "DBMail/%s" % get_version(),
    }

    kwargs.update({
        'user': settings.SMSAERO_LOGIN,
        'password': settings.SMSAERO_MD5_PASSWORD,
        'from': kwargs.pop('sms_from', settings.SMSAERO_FROM),
        'to': sms_to.replace('+', ''),
        'text': from_unicode(sms_body),
        'answer': 'json',
    })

    http = HTTPConnection(kwargs.pop("api_url", "gate.smsaero.ru"))
    http.request("GET", "/send/?" + urlencode(kwargs), headers=headers)
    response = http.getresponse()

    if response.status != 200:
        raise AeroSmsError(response.reason)

    read = response.read().decode(response.headers.get_content_charset())
    data = json.loads(read)

    status = None
    if 'result' in data:
        status = data['result']

    sms_id = None
    if 'id' in data:
        sms_id = data['id']

    if sms_id and status == 'accepted':
        return True
    return False



# -*- encoding: utf-8 -*-

import datetime
import pickle
import uuid
import os
import re

from django.utils.translation import ugettext_lazy as _
from django.utils.html import strip_tags
from django.utils.timezone import now
from django.core.cache import cache
from django.utils import timezone
from django.conf import settings
from django.db import models
from django import VERSION

from dbmail.defaults import (
    PRIORITY_STEPS, UPLOAD_TO, DEFAULT_CATEGORY, AUTH_USER_MODEL,
    DEFAULT_FROM_EMAIL, DEFAULT_PRIORITY, CACHE_TTL,
    BACKEND, _BACKEND, BACKENDS_MODEL_CHOICES, MODEL_HTMLFIELD,
    MODEL_SUBSCRIPTION_DATA_FIELD
)

from dbmail import initial_signals, import_by_string
from dbmail import python_2_unicode_compatible
from dbmail.utils import premailer_transform


HTMLField = import_by_string(MODEL_HTMLFIELD)
SubscriptionDataField = import_by_string(MODEL_SUBSCRIPTION_DATA_FIELD)


def _upload_mail_file(instance, filename):
    if instance is not None:
        ext = filename.split('.')[-1]
        filename = "%s.%s" % (str(uuid.uuid4()), ext)
        return os.path.join(UPLOAD_TO, filename)


@python_2_unicode_compatible
class MailCategory(models.Model):
    name = models.CharField(_('Category'), max_length=25, unique=True)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail category')
        verbose_name_plural = _('Mail categories')


@python_2_unicode_compatible
class MailBaseTemplate(models.Model):
    name = models.CharField(_('Name'), max_length=100, unique=True)
    message = HTMLField(
        _('Body'), help_text=_(
            'Basic template for mail messages. {{content}} tag for msg.'))
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail base template')
        verbose_name_plural = _('Mail base templates')


@python_2_unicode_compatible
class MailFromEmailCredential(models.Model):
    host = models.CharField(_('Host'), max_length=50)
    port = models.PositiveIntegerField(_('Port'))
    username = models.CharField(
        _('Username'), max_length=50, null=True, blank=True)
    password = models.CharField(
        _('Password'), max_length=50, null=True, blank=True)
    use_tls = models.BooleanField(_('Use TLS'), default=False)
    fail_silently = models.BooleanField(_('Fail silently'), default=False)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def _clean_cache(self):
        for obj in MailFromEmail.objects.filter(credential=self):
            obj._clean_template_cache()

    def delete(self, using=None):
        self._clean_cache()
        super(MailFromEmailCredential, self).delete(using)

    def save(self, *args, **kwargs):
        super(MailFromEmailCredential, self).save(*args, **kwargs)
        self._clean_cache()

    def __str__(self):
        return '%s/%s' % (self.username, self.host)

    class Meta:
        verbose_name = _('Mail auth settings')
        verbose_name_plural = _('Mail auth settings')


@python_2_unicode_compatible
class MailFromEmail(models.Model):
    name = models.CharField(_('Name'), max_length=100)
    email = models.CharField(
        _('Email'), max_length=75, unique=True,
        help_text=_('For sms/tts/push you must specify name or number'))
    credential = models.ForeignKey(
        MailFromEmailCredential, verbose_name=_('Auth credentials'),
        blank=True, null=True, default=None)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    @property
    def get_mail_from(self):
        return u'%s <%s>' % (self.name, self.email)

    def _clean_template_cache(self):
        MailTemplate.clean_cache(from_email=self)

    def get_auth(self):
        if self.credential:
            return dict(
                host=self.credential.host,
                port=self.credential.port,
                username=str(self.credential.username),
                password=str(self.credential.password),
                use_tls=self.credential.use_tls,
                fail_silently=self.credential.fail_silently
            )

    def delete(self, using=None):
        self._clean_template_cache()
        super(MailFromEmail, self).delete(using)

    def save(self, *args, **kwargs):
        super(MailFromEmail, self).save(*args, **kwargs)
        self._clean_template_cache()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail from')
        verbose_name_plural = _('Mail from')


@python_2_unicode_compatible
class MailBcc(models.Model):
    email = models.EmailField(_('Email'), unique=True)
    is_active = models.BooleanField(_('Is active'), default=True)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def __clean_cache(self):
        MailTemplate.clean_cache(bcc_email=self)

    def save(self, *args, **kwargs):
        super(MailBcc, self).save(*args, **kwargs)
        self.__clean_cache()

    def delete(self, using=None):
        self.__clean_cache()
        super(MailBcc, self).delete(using)

    def __str__(self):
        return self.email

    class Meta:
        verbose_name = _('Mail Bcc')
        verbose_name_plural = _('Mail Bcc')


@python_2_unicode_compatible
class MailTemplate(models.Model):
    name = models.CharField(_('Template name'), max_length=100, db_index=True)
    subject = models.CharField(_('Subject'), max_length=100)
    from_email = models.ForeignKey(
        MailFromEmail, null=True, blank=True,
        verbose_name=_('Message from'), default=DEFAULT_FROM_EMAIL,
        help_text=_('If not specified, then used default.'),
        on_delete=models.SET_NULL)
    bcc_email = models.ManyToManyField(
        MailBcc, verbose_name=_('Bcc'), blank=True,
        help_text='Blind carbon copy')
    message = HTMLField(_('Body'))
    slug = models.SlugField(
        _('Slug'), unique=True,
        help_text=_('Unique slug to use in code.'))
    num_of_retries = models.PositiveIntegerField(
        _('Number of retries'), default=1)
    priority = models.SmallIntegerField(
        _('Priority'), default=DEFAULT_PRIORITY, choices=PRIORITY_STEPS)
    is_html = models.BooleanField(
        _('Is html'), default=True,
        help_text=_('For sms/tts/push must be text not html'))
    is_admin = models.BooleanField(_('For admin'), default=False)
    is_active = models.BooleanField(_('Is active'), default=True)
    enable_log = models.BooleanField(_('Logging enabled'), default=True)
    category = models.ForeignKey(
        MailCategory, null=True, blank=True,
        verbose_name=_('Category'), default=DEFAULT_CATEGORY)
    base = models.ForeignKey(
        MailBaseTemplate, null=True, blank=True,
        verbose_name=_('Basic template'))
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)
    context_note = models.TextField(
        _('Context note'), null=True, blank=True,
        help_text=_(
            'This is simple note field for context variables with description.'
        )
    )
    interval = models.PositiveIntegerField(
        _('Send interval'), null=True, blank=True,
        help_text=_(
            """
            Specify interval to send messages after sometime.
            Interval must be set in the seconds.
            """
        ))

    def _clean_cache(self):
        cache.delete(self.slug, version=1)

    @classmethod
    def clean_cache(cls, **kwargs):
        for template in cls.objects.filter(**kwargs):
            template._clean_cache()

    def _clean_non_html(self):
        if not self.is_html:
            self.message = strip_tags(self.message)
            if hasattr(settings, 'MODELTRANSLATION_LANGUAGES'):
                for lang in settings.MODELTRANSLATION_LANGUAGES:
                    message = strip_tags(getattr(self, 'message_%s' % lang))
                    if message:
                        setattr(self, 'message_%s' % lang, message)

    def _premailer_transform(self):
        if self.is_html:
            self.message = premailer_transform(self.message)

    @classmethod
    def get_template(cls, slug):
        obj = cache.get(slug, version=1)

        if obj is not None:
            return obj

        obj = cls.objects.select_related('from_email', 'base').get(slug=slug)
        bcc_list = [o.email for o in obj.bcc_email.filter(is_active=1)]
        files_list = list(obj.files.all())
        auth_credentials = obj.from_email and obj.from_email.get_auth()

        obj.__dict__['bcc_list'] = bcc_list
        obj.__dict__['files_list'] = files_list
        obj.__dict__['auth_credentials'] = auth_credentials

        cache.set(slug, obj, timeout=CACHE_TTL, version=1)

        return obj

    def save(self, *args, **kwargs):
        self._premailer_transform()
        self._clean_non_html()
        self.slug = re.sub(r'[^0-9a-zA-Z._-]', '', self.slug)
        super(MailTemplate, self).save(*args, **kwargs)
        self._clean_cache()

    def delete(self, using=None):
        self._clean_cache()
        super(MailTemplate, self).delete(using)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail template')
        verbose_name_plural = _('Mail templates')


@python_2_unicode_compatible
class MailFile(models.Model):
    template = models.ForeignKey(
        MailTemplate, verbose_name=_('Template'), related_name='files')
    name = models.CharField(_('Name'), max_length=100)
    filename = models.FileField(_('File'), upload_to=_upload_mail_file)

    def _clean_cache(self):
        MailTemplate.clean_cache(pk=self.template.pk)

    def save(self, *args, **kwargs):
        super(MailFile, self).save(*args, **kwargs)
        self._clean_cache()

    def delete(self, using=None):
        self._clean_cache()
        super(MailFile, self).delete(using)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail file')
        verbose_name_plural = _('Mail files')


@python_2_unicode_compatible
class MailLogException(models.Model):
    cache_key = 'ignored_exceptions'
    cache_version = 1

    name = models.CharField(_('Exception'), max_length=150, unique=True)
    ignore = models.BooleanField(_('Ignore'), default=False)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        super(MailLogException, self).save(*args, **kwargs)
        self._clean_cache()

    @classmethod
    def get_ignored_exceptions(cls):
        obj = cache.get(cls.cache_key, version=cls.cache_version)
        if obj is not None:
            return obj
        obj = cls.objects.values_list('name', flat=True).filter(ignore=True)
        cache.set(
            cls.cache_key, obj, timeout=CACHE_TTL, version=cls.cache_version)
        return obj

    @classmethod
    def get_or_create(cls, name):
        obj, created = MailLogException.objects.get_or_create(name=name)
        if created is True:
            cls._clean_cache()
        return obj

    @classmethod
    def _clean_cache(cls):
        cache.delete(cls.cache_key, version=cls.cache_version)
        cls.get_ignored_exceptions()

    class Meta:
        verbose_name = _('Mail Exception')
        verbose_name_plural = _('Mail Exception')


@python_2_unicode_compatible
class MailLog(models.Model):
    is_sent = models.BooleanField(_('Is sent'), default=True, db_index=True)
    template = models.ForeignKey(MailTemplate, verbose_name=_('Template'))
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    user = models.ForeignKey(
        AUTH_USER_MODEL, verbose_name=_('User'),
        null=True, blank=True)
    error_message = models.TextField(_('Error message'), null=True, blank=True)
    error_exception = models.ForeignKey(
        MailLogException, null=True, blank=True, verbose_name=_('Exception'))
    num_of_retries = models.PositiveIntegerField(
        _('Number of retries'), default=1)
    log_id = models.CharField(
        _('Log ID'), max_length=60, editable=False, db_index=True)
    backend = models.CharField(
        _('Backend'), max_length=25, editable=False, db_index=True,
        choices=sorted(list(BACKEND.items())), default='mail')
    provider = models.CharField(
        _('Provider'), max_length=250, editable=False, db_index=True,
        default=None, null=True, blank=True)

    @staticmethod
    def store_email_log(log, email_list, mail_type):
        if log and email_list:
            for email in email_list:
                MailLogEmail.objects.create(
                    log=log, email=email, mail_type=mail_type
                )

    @classmethod
    def store(cls, to, cc, bcc, is_sent, template,
              user, num, msg='', ex=None, log_id=None,
              backend=None, provider=None):
        if ex is not None:
            ex = MailLogException.get_or_create(ex)

        log = cls.objects.create(
            template=template, is_sent=is_sent, user=user,
            log_id=log_id, num_of_retries=num, error_message=msg,
            error_exception=ex, backend=_BACKEND.get(backend, backend),
            provider=provider
        )
        cls.store_email_log(log, to, 'to')
        cls.store_email_log(log, cc, 'cc')
        cls.store_email_log(log, bcc, 'bcc')

    @classmethod
    def cleanup(cls, days=7):
        date = now() - datetime.timedelta(days=days)
        cls.objects.filter(created__lte=date).delete()

    def __str__(self):
        return self.template.name

    class Meta:
        verbose_name = _('Mail log')
        verbose_name_plural = _('Mail logs')


@python_2_unicode_compatible
class MailLogEmail(models.Model):
    log = models.ForeignKey(MailLog)
    email = models.CharField(_('Recipient'), max_length=350)
    mail_type = models.CharField(_('Mail type'), choices=(
        ('cc', 'CC'),
        ('bcc', 'BCC'),
        ('to', 'TO'),
    ), max_length=3)

    def __str__(self):
        return self.email

    class Meta:
        verbose_name = _('Mail log email')
        verbose_name_plural = _('Mail log emails')


@python_2_unicode_compatible
class MailGroup(models.Model):
    name = models.CharField(_('Group name'), max_length=100)
    slug = models.SlugField(_('Slug'), unique=True)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def clean_cache(self):
        cache.delete(self.slug, version=4)

    @classmethod
    def get_emails(cls, slug):
        emails = cache.get(slug, version=4)

        if emails is not None:
            return emails

        emails = MailGroupEmail.objects.values_list(
            'email', flat=True).filter(group__slug=slug)

        cache.set(slug, emails, timeout=CACHE_TTL, version=4)
        return emails

    def save(self, *args, **kwargs):
        self.slug = re.sub(r'[^0-9a-zA-Z._-]', '', self.slug)
        super(MailGroup, self).save(*args, **kwargs)
        self.clean_cache()

    def delete(self, using=None):
        self.clean_cache()
        super(MailGroup, self).delete(using)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail group')
        verbose_name_plural = _('Mail groups')


@python_2_unicode_compatible
class MailGroupEmail(models.Model):
    name = models.CharField(_('Username'), max_length=100)
    email = models.CharField(
        _('Email'), max_length=75,
        help_text='For sms/tts you must specify number')
    group = models.ForeignKey(
        MailGroup, verbose_name=_('Group'), related_name='emails')

    def save(self, *args, **kwargs):
        super(MailGroupEmail, self).save(*args, **kwargs)
        self.group.clean_cache()

    def delete(self, using=None):
        self.group.clean_cache()
        super(MailGroupEmail, self).delete(using)

    def __str__(self):
        return self.email

    class Meta:
        verbose_name = _('Mail group email')
        verbose_name_plural = _('Mail group emails')
        unique_together = (('email', 'group',),)


@python_2_unicode_compatible
class Signal(models.Model):
    SIGNALS = (
        'pre_save',
        'post_save',
        'pre_delete',
        'post_delete',
        'm2m_changed',
    )
    name = models.CharField(_('Name'), max_length=100)
    model = models.ForeignKey(
        'contenttypes.ContentType', verbose_name=_('Model'))
    signal = models.CharField(
        _('Signal'), choices=zip(SIGNALS, SIGNALS),
        max_length=15, default='post_save')
    template = models.ForeignKey(MailTemplate, verbose_name=_('Template'))
    group = models.ForeignKey(
        MailGroup, verbose_name=_('Email group'), blank=True, null=True,
        help_text=_('You can use group email or rules for recipients.'))
    rules = models.TextField(
        help_text=_(
            'Template should return email to send message. Example:'
            '{% if instance.is_active %}{{ instance.email }}{% endif %}.'
            'You can return a multiple emails separated by commas.'
        ), default='{{ instance.email }}', verbose_name=_('Rules'),
        null=True, blank=True
    )
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)
    is_active = models.BooleanField(_('Is active'), default=True)
    receive_once = models.BooleanField(
        _('Receive once'), default=True,
        help_text=_('Signal will be receive and send once for new db row.'))
    interval = models.PositiveIntegerField(
        _('Send interval'), null=True, blank=True,
        help_text=_(
            'Specify interval to send messages after sometime. '
            'That very helpful for mailing on enterprise products.'
            'Interval must be set in the seconds.'
        ))
    update_model = models.BooleanField(
        _('Update model state'), default=False,
        help_text=_(
            """
            If you are using interval and want to update object state,
            you can use this flag and refer to the variable
            {{current_instance}}
            """))

    def is_sent(self, pk):
        if pk is not None:
            if self.receive_once is True:
                return SignalLog.objects.filter(
                    model=self.model, model_pk=pk, signal=self).exists()
            return False

    def mark_as_sent(self, pk):
        if pk is not None:
            SignalLog.objects.create(
                model=self.model, model_pk=pk, signal=self
            )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail signal')
        verbose_name_plural = _('Mail signals')


@python_2_unicode_compatible
class SignalLog(models.Model):
    model = models.ForeignKey('contenttypes.ContentType')
    model_pk = models.BigIntegerField()
    signal = models.ForeignKey(Signal)
    created = models.DateTimeField(_('Created'), auto_now_add=True)

    def __str__(self):
        return self.signal.name

    class Meta:
        verbose_name = _('Signal log')
        verbose_name_plural = _('Signal logs')


class SignalDeferredDispatch(models.Model):
    args = models.TextField()
    kwargs = models.TextField()
    params = models.TextField()
    eta = models.DateTimeField(db_index=True)
    done = models.NullBooleanField(default=None)
    created = models.DateTimeField(auto_now_add=True)

    def run_task(self):
        if self.done is False:
            import tasks

            tasks.deferred_signal.apply_async(
                args=pickle.loads(self.args),
                kwargs=pickle.loads(self.kwargs),
                **pickle.loads(self.params)
            )

    @classmethod
    def add_task(cls, args, kwargs, params, interval):
        eta = now() + datetime.timedelta(seconds=interval)
        dump = pickle.dumps
        return cls.objects.create(
            args=dump(args), kwargs=dump(kwargs),
            params=dump(params), eta=eta
        )

    class Meta:
        if VERSION >= (1, 5):
            index_together = (('eta', 'done'),)


@python_2_unicode_compatible
class ApiKey(models.Model):
    name = models.CharField(_('Name'), max_length=25)
    api_key = models.CharField(_('Api key'), max_length=32, unique=True)
    is_active = models.BooleanField(_('Is active'), default=True)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def _clean_cache(self):
        cache.delete(self.api_key)

    @classmethod
    def clean_cache(cls):
        for api in cls.objects.all():
            api._clean_cache()

    def save(self, *args, **kwargs):
        super(ApiKey, self).save(*args, **kwargs)
        self._clean_cache()

    def delete(self, using=None):
        self._clean_cache()
        super(ApiKey, self).delete(using)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail API')
        verbose_name_plural = _('Mail API')


@python_2_unicode_compatible
class MailLogTrack(models.Model):
    mail_log = models.ForeignKey(MailLog, verbose_name=_('Log'))
    counter = models.PositiveIntegerField(_('Counter'), default=0)
    ip = models.GenericIPAddressField(_('IP'))
    ua = models.CharField(
        _('User Agent'), max_length=350, blank=True, null=True)

    ua_os = models.CharField(_('OS'), max_length=100, blank=True, null=True)
    ua_os_version = models.CharField(
        _('OS version'), max_length=100, blank=True, null=True)

    ua_dist = models.CharField(
        _('Dist name'), max_length=20, blank=True, null=True)
    ua_dist_version = models.CharField(
        _('Dist version'), max_length=100, blank=True, null=True)
    ua_browser = models.CharField(
        _('Browser'), max_length=100, blank=True, null=True)
    ua_browser_version = models.CharField(
        _('Browser version'), max_length=20, blank=True, null=True)
    ip_area_code = models.CharField(
        _('Area code'), max_length=255, blank=True, null=True)
    ip_city = models.CharField(
        _('City'), max_length=255, blank=True, null=True)
    ip_country_code = models.CharField(
        _('Country code'), max_length=255, blank=True, null=True)
    ip_country_code3 = models.CharField(
        _('Country code3'), max_length=255, blank=True, null=True)
    ip_country_name = models.CharField(
        _('Country name'), max_length=255, blank=True, null=True)
    ip_dma_code = models.CharField(
        _('Dma code'), max_length=255, blank=True, null=True)
    ip_latitude = models.CharField(
        _('Latitude'), max_length=255, blank=True, null=True)
    ip_longitude = models.CharField(
        _('Longitude'), max_length=255, blank=True, null=True)
    ip_postal_code = models.CharField(
        _('Postal code'), max_length=255, blank=True, null=True)
    ip_region = models.CharField(
        _('Region'), max_length=255, blank=True, null=True)

    is_read = models.BooleanField(_('Is read'), default=False)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def __str__(self):
        return self.mail_log.template.name

    class Meta:
        verbose_name = _('Mail Tracking')
        verbose_name_plural = _('Mail Tracking')

    def detect_ua(self):
        try:
            if self.ua and self.counter == 0:
                import httpagentparser

                data = httpagentparser.detect(self.ua)
                get = lambda b, k: data.get(b, {}).get(k, '')

                self.ua_os = get('os', 'name')
                self.ua_os_version = get('os', 'version')

                self.ua_dist = get('platform', 'name')
                self.ua_dist_version = get('platform', 'version')

                self.ua_browser = get('browser', 'name')
                self.ua_browser_version = get('browser', 'version')

        except ImportError:
            pass

    def detect_geo(self):
        if self.ip and self.counter == 0:
            from django.contrib.gis.geoip import GeoIP, GeoIPException

            try:
                g = GeoIP()
                info = g.city(self.ip) or dict()
                for (k, v) in info.items():
                    setattr(self, 'ip_%s' % k, v)
            except GeoIPException:
                pass

    def detect_open(self):
        if self.counter == 0:
            self.counter = 1
        elif self.counter >= 1:
            self.counter += 1

    def save(self, *args, **kwargs):
        self.detect_ua()
        self.detect_geo()
        self.detect_open()
        super(MailLogTrack, self).save(*args, **kwargs)


class MailSubscriptionAbstract(models.Model):
    title = models.CharField(null=True, max_length=350, blank=True)

    user = models.ForeignKey(
        AUTH_USER_MODEL, verbose_name=_('User'), null=True, blank=True)
    backend = models.CharField(
        _('Backend'), choices=BACKENDS_MODEL_CHOICES, max_length=50,
        default=BACKEND.get('mail'))
    start_hour = models.CharField(
        _('Start hour'), default='00:00', max_length=5)
    end_hour = models.CharField(_('End hour'), default='23:59', max_length=5)
    is_enabled = models.BooleanField(
        _('Is enabled'), default=True, db_index=True)
    is_checked = models.BooleanField(
        _('Is checked'), default=False, db_index=True)
    defer_at_allowed_hours = models.BooleanField(
        _('Defer at allowed hours'), default=False)
    address = models.CharField(
        _('Address'), max_length=350, db_index=True,
        help_text=_('Must be phone number/email/token'))
    data = SubscriptionDataField(null=True, blank=True)

    def send_confirmation_link(
            self, slug='subs-confirmation', *args, **kwargs):

        from dbmail import db_sender

        kwargs['backend'] = self.backend
        db_sender(slug, self.address, *args, **kwargs)

    @staticmethod
    def get_now():
        d = timezone.now()
        if d.tzinfo:
            return timezone.localtime(timezone.now())
        return d

    @staticmethod
    def get_current_hour():
        current = MailSubscriptionAbstract.get_now()
        return datetime.timedelta(hours=current.hour, minutes=current.minute)

    @staticmethod
    def convert_to_date(value):
        hour, minute = value.split(':')
        return datetime.timedelta(hours=int(hour), minutes=int(minute))

    @classmethod
    def mix_hour_with_date(cls, value):
        return datetime.datetime.strptime(
            cls.get_now().strftime('%Y-%m-%d ') + value, '%Y-%m-%d %H:%M')

    @classmethod
    def get_notification_list(cls, user_id, **kwargs):
        kwargs.update({
            'is_enabled': True,
            'is_checked': True,
        })
        if user_id is not None:
            kwargs.update({
                'user_id': user_id,
            })
        return cls.objects.filter(**kwargs)

    @classmethod
    def notify(cls, slug, user_id=None, sub_filter=None, **kwargs):

        from dbmail import db_sender

        now_hour = cls.get_current_hour()

        context_dict = kwargs.pop('context', {})
        context_instance = kwargs.pop('context_instance', None)

        sub_filter = sub_filter if isinstance(sub_filter, dict) else {}

        for method_id in cls.get_notification_list(
                user_id, **sub_filter).values_list('pk', flat=True):

            method = cls.objects.get(pk=method_id)
            method_kwargs = kwargs.copy()

            method_kwargs['send_at_date'] = None
            start_hour = cls.convert_to_date(method.start_hour)
            end_hour = cls.convert_to_date(method.end_hour)

            if not (start_hour <= now_hour <= end_hour):
                if (method.defer_at_allowed_hours and
                        method_kwargs['use_celery']):

                    method_kwargs['send_at_date'] = cls.mix_hour_with_date(
                        method.start_hour)
                else:
                    continue
            method_kwargs['backend'] = method.backend

            extra_slug = '%s-%s' % (slug, method.get_short_type())
            use_slug = slug

            method_kwargs = method.update_notify_kwargs(**method_kwargs)
            try:
                if MailTemplate.get_template(slug=extra_slug):
                    use_slug = extra_slug
            except MailTemplate.DoesNotExist:
                pass
            db_sender(use_slug, method.address, context_dict,
                      context_instance, **method_kwargs)

    def update_notify_kwargs(self, **kwargs):
        return kwargs

    def get_short_type(self):
        return self.backend.split('.')[-1]

    class Meta:
        abstract = True


@python_2_unicode_compatible
class MailSubscription(MailSubscriptionAbstract):
    class Meta:
        verbose_name = _('Mail Subscription')
        verbose_name_plural = _('Mail Subscriptions')

    def __str__(self):
        if self.user:
            return self.user.username
        return self.address


'''
class MailSubscriptionGroup(models.Model):
    group = models.CharField(max_length=100, unique=True)


class MailNotification(models.Model):
    group = models.ForeignKey(AbstractMailSubscriptionGroup)
    notify = models.ManyToManyField(MailSubscription)

    @classmethod
    def notify(cls, user, mail_slug, group, **kwargs):
        from dbmail import send_db_subscription

        for notify in cls.objects.filter(notify__user=user, group=group):
            send_db_subscription(
                mail_slug, user.pk, {'pk': notify.pk}, **kwargs)
'''

if VERSION < (1, 7):
    initial_signals()



"""SendInBlue mail provider."""

import base64
import email

from mailin import Mailin


class SendInBlueError(Exception):
    """Custom exception."""

    pass


def email_list_to_email_dict(email_list):
    """Convert a list of email to a dict of email."""
    if email_list is None:
        return {}
    result = {}
    for value in email_list:
        realname, address = email.utils.parseaddr(value)
        result[address] = realname if realname and address else address
    return result


def email_address_to_list(email_address):
    """Convert an email address to a list."""
    realname, address = email.utils.parseaddr(email_address)
    return (
        [address, realname] if realname and address else
        [email_address, email_address]
    )


def send(sender_instance):
    """Send a transactional email using SendInBlue API.

    Site: https://www.sendinblue.com
    API: https://apidocs.sendinblue.com/
    """
    m = Mailin(
        "https://api.sendinblue.com/v2.0",
        sender_instance._kwargs.get("api_key")
    )
    data = {
        "to": email_list_to_email_dict(sender_instance._recipient_list),
        "cc": email_list_to_email_dict(sender_instance._cc),
        "bcc": email_list_to_email_dict(sender_instance._bcc),
        "from": email_address_to_list(sender_instance._from_email),
        "subject": sender_instance._subject,
    }
    if sender_instance._template.is_html:
        data.update({
            "html": sender_instance._message,
            "headers": {"Content-Type": "text/html; charset=utf-8"}
        })
    else:
        data.update({"text": sender_instance._message})
    if "attachments" in sender_instance._kwargs:
        data["attachment"] = {}
        for attachment in sender_instance._kwargs["attachments"]:
            data["attachment"][attachment[0]] = base64.b64encode(attachment[1])
    result = m.send_email(data)
    if result["code"] != "success":
        raise SendInBlueError(result["message"])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dbmail', '0011_auto_20160803_1024'),
    ]

    operations = [
        migrations.AlterField(
            model_name='maillogemail',
            name='email',
            field=models.CharField(max_length=350, verbose_name='Recipient'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.core import signing
from celery import task

from dbmail.defaults import SEND_RETRY_DELAY, SEND_RETRY, SEND_MAX_TIME, DEBUG
from dbmail.utils import get_ip


@task(name='dbmail.db_sender', default_retry_delay=SEND_RETRY_DELAY)
def db_sender(*args, **kwargs):
    from dbmail import import_module

    retry_delay = kwargs.pop('retry_delay', SEND_RETRY_DELAY)
    time_limit = kwargs.pop('time_limit', SEND_MAX_TIME)
    max_retries = kwargs.pop('max_retries', SEND_RETRY)
    backend = import_module(kwargs.get('backend'))
    retry = kwargs.pop('retry', True)

    try:
        if DEBUG is True:
            return backend.SenderDebug(*args, **kwargs).send(is_celery=True)
        return backend.Sender(*args, **kwargs).send(is_celery=True)
    except Exception as exc:
        if retry is True and max_retries:
            raise db_sender.retry(
                retry=retry, max_retries=max_retries,
                countdown=retry_delay, exc=exc,
                time_limit=time_limit,
            )
        raise


@task(name='dbmail.subscription')
def db_subscription(*args, **kwargs):
    from dbmail import import_by_string
    from dbmail.defaults import MAIL_SUBSCRIPTION_MODEL

    MailSubscription = import_by_string(MAIL_SUBSCRIPTION_MODEL)

    MailSubscription.notify(*args, **kwargs)


@task(name='dbmail.signal_receiver')
def signal_receiver(*args, **kwargs):
    from dbmail.signals import SignalReceiver

    SignalReceiver(*args, **kwargs).run()
    if len(args):
        return args[0]._meta.module_name


@task(name='dbmail.deferred_signal')
def deferred_signal(*args, **kwargs):
    from dbmail.signals import SignalReceiver

    SignalReceiver(*args, **kwargs).run_deferred()
    return 'OK'


@task(name='dbmail.mail_track')
def mail_track(http_meta, encrypted):
    from dbmail.models import MailLogTrack, MailLog

    class Request(object):
        META = http_meta

    try:
        request = Request()

        mail_log_id = signing.loads(encrypted)
        mail_log = MailLog.objects.get(log_id=mail_log_id)

        track_log = MailLogTrack.objects.filter(mail_log=mail_log)
        if not track_log.exists():
            MailLogTrack.objects.create(
                mail_log=mail_log,
                ip=get_ip(request),
                ua=request.META.get('HTTP_USER_AGENT'),
                is_read=True,
            )
        else:
            track_log[0].save()
    except (signing.BadSignature, MailLog.DoesNotExist):
        pass
    except Exception as exc:
        raise mail_track.retry(
            retry=True, max_retries=SEND_RETRY,
            countdown=SEND_RETRY_DELAY, exc=exc,
        )



# -*- encoding: utf-8 -*-

from httplib import HTTPConnection, HTTPSConnection
from urlparse import urlparse
from json import dumps

from django.conf import settings


class HTTPError(Exception):
    pass


def send(hook_url, message, **kwargs):
    headers = {
        "Content-type": "application/json"
    }

    http_key = kwargs.pop("http_key", None)
    if not http_key and hasattr(settings, 'HTTP_KEY'):
        http_key = settings.HTTP_KEY
    headers["Authorization"] = "key={}".format(http_key)

    kwargs["msg"] = message

    up = urlparse(hook_url)
    if up.scheme == 'https':
        http = HTTPSConnection(up.netloc)
    else:
        http = HTTPConnection(up.netloc)

    http.request(
        "POST", up.path,
        headers=headers,
        body=dumps(kwargs))
    response = http.getresponse()

    if response.status != 200:
        raise HTTPError(response.reason)
    return True



# -*- encoding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _
from django.apps import AppConfig


class DBMailConfig(AppConfig):
    name = 'dbmail'
    verbose_name = _('DB Mailer')

    def ready(self):
        from dbmail import initial_signals

        initial_signals()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dbmail', '0007_auto_20150708_2016'),
    ]

    operations = [

        migrations.AlterField(
            model_name='mailbcc',
            name='email',
            field=models.EmailField(unique=True, max_length=254, verbose_name='Email'),
        ),
        migrations.AlterField(
            model_name='mailfromemail',
            name='email',
            field=models.CharField(help_text='For sms/tts/push you must specify name or number', unique=True, max_length=75, verbose_name='Email'),
        ),
        migrations.AlterField(
            model_name='maillog',
            name='log_id',
            field=models.CharField(verbose_name='Log ID', max_length=60, editable=False, db_index=True),
        ),
        migrations.AlterField(
            model_name='maillog',
            name='provider',
            field=models.CharField(default=None, editable=False, max_length=250, blank=True, null=True, verbose_name='Provider', db_index=True),
        ),
        migrations.AlterField(
            model_name='maillogemail',
            name='email',
            field=models.CharField(max_length=75, verbose_name='Recipient'),
        ),
        migrations.AlterField(
            model_name='mailtemplate',
            name='bcc_email',
            field=models.ManyToManyField(help_text=b'Blind carbon copy', to='dbmail.MailBcc', verbose_name='Bcc', blank=True),
        ),
        migrations.AlterField(
            model_name='mailtemplate',
            name='is_html',
            field=models.BooleanField(default=True, help_text='For sms/tts/push must be text not html', verbose_name='Is html'),
        ),
    ]



from django.core.management.base import BaseCommand
from django.utils.timezone import now

from dbmail.defaults import SIGNAL_DB_DEFERRED_PURGE
from dbmail.models import SignalDeferredDispatch


class Command(BaseCommand):
    def __init__(self):
        super(Command, self).__init__()
        self.signal = None

    def done(self):
        if SIGNAL_DB_DEFERRED_PURGE is True:
            self.signal.delete()
        else:
            self.signal.done = True
            self.signal.save()

    def start(self):
        self.signal.done = False
        self.signal.save()

    @staticmethod
    def signals():
        return SignalDeferredDispatch.objects.filter(
            done__isnull=True, eta__lte=now())

    def handle(self, *args, **options):
        for self.signal in self.signals():
            self.start()
            self.signal.run_task()
            self.done()



# -*- encoding: utf-8 -*-

from django.conf.urls import url

from dbmail.views import (
    send_by_dbmail, mail_read_tracker,
    SafariPushPackagesView, SafariSubscriptionView, SafariLogView,
    PushSubscriptionView
)

urlpatterns = [
    url(r'^api/', send_by_dbmail, name='db-mail-api'),
    url(r'^mail_read_tracker/(.*?)/$',
        mail_read_tracker, name='db-mail-tracker'),

    url(r'^safari/v(?P<version>[0-9]{1})/pushPackages/(?P<site_pid>[.\w-]+)/?',
        SafariPushPackagesView.as_view()),
    url(r'^safari/v(?P<version>[0-9]{1})/devices/'
        r'(?P<device_token>[.\w-]+)/registrations/(?P<site_pid>[.\w-]+)/?',
        SafariSubscriptionView.as_view()),
    url(r'^safari/v(?P<version>[0-9]{1})/log/?', SafariLogView.as_view()),

    url(r'^(?P<reg_type>web-push|mobile)/subscribe/',
        PushSubscriptionView.as_view(), name='push-subscribe'),
    url(r'^(?P<reg_type>web-push|mobile)/unsubscribe/',
        PushSubscriptionView.as_view(), name='push-unsubscribe'),
]



# -*- encoding: utf-8 -*-

from django.conf import settings
from Pubnub import Pubnub


class PushOverError(Exception):
    pass


def send(channel, message, **kwargs):
    """
    Site: http://www.pubnub.com/
    API: https://www.mashape.com/pubnub/pubnub-network
    Desc: real-time browser notifications

    Installation and usage:
    pip install -U pubnub
    Tests for browser notification http://127.0.0.1:8000/browser_notification/
    """

    pubnub = Pubnub(
        publish_key=settings.PUBNUB_PUB_KEY,
        subscribe_key=settings.PUBNUB_SUB_KEY,
        secret_key=settings.PUBNUB_SEC_KEY,
        ssl_on=kwargs.pop('ssl_on', False), **kwargs)
    return pubnub.publish(channel=channel, message={"text": message})



# -*- encoding: utf-8 -*-

from httplib import HTTPConnection
from urllib import urlencode
from base64 import b64encode

from django.conf import settings

from dbmail.providers.prowl.push import from_unicode
from dbmail import get_version


class IQSMSError(Exception):
    pass


def send(sms_to, sms_body, **kwargs):
    """
    Site: http://iqsms.ru/
    API: http://iqsms.ru/api/
    """
    headers = {
        "User-Agent": "DBMail/%s" % get_version(),
        'Authorization': 'Basic %s' % b64encode(
            "%s:%s" % (
                settings.IQSMS_API_LOGIN, settings.IQSMS_API_PASSWORD
            )).decode("ascii")
    }

    kwargs.update({
        'phone': sms_to,
        'text': from_unicode(sms_body),
        'sender': kwargs.pop('sms_from', settings.IQSMS_FROM)
    })

    http = HTTPConnection(kwargs.pop("api_url", "gate.iqsms.ru"))
    http.request("GET", "/send/?" + urlencode(kwargs), headers=headers)
    response = http.getresponse()

    if response.status != 200:
        raise IQSMSError(response.reason)

    body = response.read().strip()
    if '=accepted' not in body:
        raise IQSMSError(body)

    return int(body.split('=')[0])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dbmail', '0010_auto_20160728_1645'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mailsubscription',
            name='address',
            field=models.CharField(help_text='Must be phone number/email/token', max_length=350, verbose_name='Address', db_index=True),
        ),
        migrations.AlterField(
            model_name='mailsubscription',
            name='title',
            field=models.CharField(max_length=350, null=True, blank=True),
        ),
    ]



# -*- encoding: utf-8 -*-

from httplib import HTTPSConnection
from urllib import urlencode

from dbmail.providers.prowl.push import from_unicode
from dbmail import get_version


class BoxcarError(Exception):
    pass


def send(token, title, **kwargs):
    """
    Site: https://boxcar.io/
    API: http://help.boxcar.io/knowledgebase/topics/48115-boxcar-api
    Desc: Best app for system administrators
    """
    headers = {
        "Content-type": "application/x-www-form-urlencoded",
        "User-Agent": "DBMail/%s" % get_version(),
    }

    data = {
        "user_credentials": token,
        "notification[title]": from_unicode(title),
        "notification[sound]": "notifier-2"
    }

    for k, v in kwargs.items():
        data['notification[%s]' % k] = from_unicode(v)

    http = HTTPSConnection(kwargs.pop("api_url", "new.boxcar.io"))
    http.request(
        "POST", "/api/notifications",
        headers=headers,
        body=urlencode(data))
    response = http.getresponse()

    if response.status != 201:
        raise BoxcarError(response.reason)
    return True



# -*- coding: utf-8 -*-

import os

from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.shortcuts import redirect, render
from django.core.urlresolvers import reverse
from django.conf.urls import url
from django.contrib import messages
from django.contrib import admin

from dbmail.models import (
    MailCategory, MailTemplate, MailLog, MailLogEmail, Signal, ApiKey, MailBcc,
    MailGroup, MailGroupEmail, MailFile, MailFromEmail, MailBaseTemplate,
    MailFromEmailCredential, MailLogTrack, MailSubscription, MailLogException
)
from dbmail import app_installed
from dbmail import get_model
from dbmail import defaults

ModelAdmin = admin.ModelAdmin

if app_installed('reversion'):
    try:
        from reversion import VersionAdmin

        ModelAdmin = VersionAdmin
    except ImportError:
        pass

if app_installed('reversion_compare'):
    try:
        from reversion_compare.admin import CompareVersionAdmin

        ModelAdmin = CompareVersionAdmin
    except ImportError:
        pass

TranslationModelAdmin = ModelAdmin

if app_installed('modeltranslation'):
    if app_installed('grappelli_modeltranslation'):
        try:
            from grappelli_modeltranslation.admin import TranslationAdmin

            class TranslationModelAdmin(ModelAdmin, TranslationAdmin):
                pass
        except ImportError:
            pass
    else:
        try:
            from modeltranslation.admin import TabbedTranslationAdmin

            class TranslationModelAdmin(TabbedTranslationAdmin, ModelAdmin):
                pass
        except ImportError:
            pass


class MailCategoryAdmin(admin.ModelAdmin):
    list_display = ('name', 'created', 'updated', 'id',)
    list_filter = ('created', 'updated',)
    search_fields = ('name',)


class MailTemplateFileAdmin(admin.TabularInline):
    model = MailFile
    extra = 1


class MailTemplateAdmin(TranslationModelAdmin):
    list_display = (
        'name', 'category', 'from_email', 'slug', 'is_admin', 'is_html',
        'enable_log', 'is_active', 'num_of_retries', 'priority',
        'created', 'updated', 'id',
    )
    list_filter = (
        'category', 'is_active', 'is_admin', 'is_html', 'priority',
        'from_email', 'created', 'updated',)
    search_fields = (
        'name', 'subject', 'slug', 'message',)
    ordering = ('-id',)
    list_editable = ('category', 'priority', 'is_active',)
    filter_horizontal = ('bcc_email',)
    list_display_links = ('name',)
    date_hierarchy = 'created'
    list_per_page = defaults.TEMPLATES_PER_PAGE
    inlines = [MailTemplateFileAdmin]
    prepopulated_fields = {'slug': ('name',)}

    class Media:
        js = (
            'dbmail/admin/js/dbmail.js',
        )

    def changelist_view(self, request, extra_context=None):
        self.change_list_template = 'dbmail/admin/change_list_link.html'
        return super(MailTemplateAdmin, self).changelist_view(
            request, extra_context=extra_context)

    def clean_cache_view(self, request):
        MailTemplate.clean_cache()
        ApiKey.clean_cache()
        messages.success(request, _('Cache was successful removed'))
        return redirect('admin:dbmail_mailtemplate_changelist')

    def send_mail_view(self, request, pk):
        from dbmail.management.commands.dbmail_test_send import send_test_msg

        if request.user.email:
            send_test_msg(
                pk, request.user.email, request.user,
                use_celery=defaults.USE_CELERY_FOR_ADMIN_TEST,
                backend=defaults.BACKEND['mail'])
            messages.success(request, _('Email was sent.'))
        else:
            messages.error(
                request, _('Set your email address in user settings.')
            )

        return redirect(
            reverse(
                'admin:dbmail_mailtemplate_change', args=(pk,),
                current_app=self.admin_site.name
            )
        )

    def get_apps_view(self, request, pk):
        apps_list = {}
        for ct in ContentType.objects.all():
            if ct.app_label in defaults.IGNORE_BROWSE_APP:
                continue
            if ct.app_label not in apps_list:
                apps_list[ct.app_label] = [ct]
            elif ct.model_class():
                apps_list[ct.app_label].append(ct)
        return render(request, 'dbmail/apps.html', {'apps_list': apps_list})

    def browse_model_fields_view(self, request, pk, app, model):
        fields = dict()
        if pk and get_model(app, model):
            for f in get_model(app, model)._meta.fields:
                try:
                    fields[f.name] = unicode(f.verbose_name)
                except NameError:
                    fields[f.name] = f.verbose_name
        return render(request, 'dbmail/browse.html', {'fields_list': fields})

    def get_urls(self):
        urls = super(MailTemplateAdmin, self).get_urls()
        admin_urls = [
            url(
                r'^(\d+)/sendmail/$',
                self.admin_site.admin_view(self.send_mail_view),
                name='send_mail_view'
            ),
            url(
                r'^(\d+)/sendmail/apps/(.*?)/(.*?)/',
                self.admin_site.admin_view(self.browse_model_fields_view),
                name='browse_model_fields_view'),
            url(
                r'^(\d+)/sendmail/apps/',
                self.admin_site.admin_view(self.get_apps_view),
                name='send_mail_apps_view'
            ),
            url(
                r'^reset/cache/',
                self.admin_site.admin_view(self.clean_cache_view),
                name='clean_cache_view'
            ),
        ]
        return admin_urls + urls

    def get_readonly_fields(self, request, obj=None):
        if obj is not None and defaults.READ_ONLY_ENABLED:
            return ['slug', 'context_note']
        return super(MailTemplateAdmin, self).get_readonly_fields(request, obj)

    def get_prepopulated_fields(self, request, obj=None):
        if obj is not None:
            return {}
        return super(MailTemplateAdmin, self).get_prepopulated_fields(
            request, obj)


class MailLogEmailInline(admin.TabularInline):
    readonly_fields = [field.name for field in MailLogEmail._meta.fields]
    model = MailLogEmail
    extra = 0

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if not request.user.is_superuser:
            return False
        return True

    def has_change_permission(self, request, obj=None):
        return request.method != 'POST'


class MailLogAdmin(admin.ModelAdmin):
    list_display = (
        'template', 'created', 'is_sent', 'num_of_retries',
        'backend', 'user', 'id',)
    list_filter = (
        'is_sent', 'backend', 'created', 'error_exception', 'template',)
    date_hierarchy = 'created'
    inlines = [MailLogEmailInline]
    search_fields = ('maillogemail__email', 'user__username', 'user__email',)

    def __init__(self, model, admin_site):
        super(MailLogAdmin, self).__init__(model, admin_site)

        self.readonly_fields = [field.name for field in model._meta.fields]
        self.readonly_model = model

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if not request.user.is_superuser:
            return False
        return True

    def has_change_permission(self, request, obj=None):
        return request.method != 'POST'


class MailGroupEmailInline(admin.TabularInline):
    model = MailGroupEmail
    extra = 1


class MailGroupAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug', 'created', 'updated', 'id',)
    list_filter = ('updated', 'created',)
    prepopulated_fields = {'slug': ('name',)}
    inlines = [MailGroupEmailInline]

    def get_readonly_fields(self, request, obj=None):
        if obj is not None and defaults.READ_ONLY_ENABLED:
            return ['slug']
        return super(MailGroupAdmin, self).get_readonly_fields(request, obj)

    def get_prepopulated_fields(self, request, obj=None):
        if obj is not None:
            return {}
        return super(MailGroupAdmin, self).get_prepopulated_fields(
            request, obj)


class MailFromEmailAdmin(admin.ModelAdmin):
    list_display = ('name', 'email', 'credential', 'created', 'updated', 'id',)
    list_filter = ('updated', 'created',)


class SignalAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'model', 'signal', 'template', 'interval', 'receive_once',
        'is_active', 'group', 'updated', 'created', 'id',)
    list_filter = (
        'signal', 'receive_once', 'is_active',
        'update_model', 'updated', 'created',)
    list_editable = ('is_active',)

    @staticmethod
    def auto_reload(request):
        if defaults.WSGI_AUTO_RELOAD is True:
            env = request.environ.get
            if env('mod_wsgi.process_group') and env('SCRIPT_FILENAME'):
                if int(env('mod_wsgi.script_reloading', 0)):
                    try:
                        if os.path.exists(env('SCRIPT_FILENAME')):
                            os.utime(env('SCRIPT_FILENAME'), None)
                    except OSError:
                        pass

        if defaults.UWSGI_AUTO_RELOAD is True:
            try:
                import uwsgi

                uwsgi.reload()
            except ImportError:
                pass

    def save_model(self, request, *args, **kwargs):
        super(SignalAdmin, self).save_model(request, *args, **kwargs)
        self.auto_reload(request)


class MailFromEmailCredentialAdmin(admin.ModelAdmin):
    list_display = (
        'host', 'port', 'username', 'use_tls',
        'fail_silently', 'updated', 'created', 'id',)
    list_filter = ('use_tls', 'fail_silently', 'updated', 'created',)


class ApiKeyAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'api_key', 'is_active', 'updated', 'created', 'id',)
    list_filter = ('is_active', 'updated', 'created',)


class MailBccAdmin(admin.ModelAdmin):
    list_display = (
        'email', 'is_active', 'updated', 'created', 'id',)
    list_filter = ('is_active', 'updated', 'created',)


class MailLogTrackAdmin(admin.ModelAdmin):
    list_display = (
        'mail_log', 'counter', 'ip', 'is_read', 'updated', 'created', 'id',)

    def __init__(self, model, admin_site):
        super(MailLogTrackAdmin, self).__init__(model, admin_site)

        self.readonly_fields = [field.name for field in model._meta.fields]
        self.readonly_model = model

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if not request.user.is_superuser:
            return False
        return True

    def has_change_permission(self, request, obj=None):
        return request.method != 'POST'


class MailBaseTemplateAdmin(TranslationModelAdmin):
    list_display = ('name', 'created', 'updated', 'id',)
    list_filter = ('created', 'updated',)
    search_fields = ('name', 'message')


class MailSubscriptionAdmin(admin.ModelAdmin):
    list_display = (
        'user', 'address', 'backend', 'start_hour', 'end_hour',
        'is_enabled', 'is_checked', 'defer_at_allowed_hours',
        'id',
    )
    list_filter = (
        'backend', 'is_enabled', 'is_checked', 'defer_at_allowed_hours')
    search_fields = (
        'user__name', 'user__email', 'address')


class MailLogExceptionAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'ignore', 'id',)
    list_filter = ('ignore',)
    search_fields = ('name',)
    readonly_fields = ('name',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if not request.user.is_superuser:
            return False
        return True


def admin_register(model):
    model_name = model.__name__
    if model_name in defaults.ALLOWED_MODELS_ON_ADMIN:
        admin_cls = globals().get('%sAdmin' % model_name)
        if admin_cls:
            admin.site.register(model, admin_cls)
        else:
            admin.site.register(model)


admin_register(MailFromEmailCredential)
admin_register(MailSubscription)
admin_register(MailBaseTemplate)
admin_register(MailLogException)
admin_register(MailFromEmail)
admin_register(MailLogTrack)
admin_register(MailCategory)
admin_register(MailTemplate)
admin_register(MailGroup)
admin_register(MailLog)
admin_register(MailBcc)
admin_register(Signal)
admin_register(ApiKey)



from django.core.management.base import BaseCommand
from dbmail.defaults import LOGS_EXPIRE_DAYS
from dbmail.models import MailLog


class Command(BaseCommand):
    def handle(self, *args, **options):
        MailLog.cleanup(days=LOGS_EXPIRE_DAYS)



# -*- coding: utf-8 -*-

from httplib import HTTPSConnection
from urllib import urlencode
from base64 import b64encode
from json import loads

from django.conf import settings

from dbmail.providers.prowl.push import from_unicode
from dbmail import get_version


class TwilioSmsError(Exception):
    pass


def send(sms_to, sms_body, **kwargs):
    """
    Site: https://www.twilio.com/
    API: https://www.twilio.com/docs/api/rest/sending-messages
    """
    headers = {
        "Content-type": "application/x-www-form-urlencoded",
        "User-Agent": "DBMail/%s" % get_version(),
        'Authorization': 'Basic %s' % b64encode(
            "%s:%s" % (
                settings.TWILIO_ACCOUNT_SID, settings.TWILIO_AUTH_TOKEN
            )).decode("ascii")

    }

    kwargs.update({
        'From': kwargs.pop('sms_from', settings.TWILIO_FROM),
        'To': sms_to,
        'Body': from_unicode(sms_body)
    })

    http = HTTPSConnection(kwargs.pop("api_url", "api.twilio.com"))
    http.request(
        "POST",
        "/2010-04-01/Accounts/%s/Messages.json" % settings.TWILIO_ACCOUNT_SID,
        headers=headers,
        body=urlencode(kwargs))

    response = http.getresponse()
    if response.status != 201:
        raise TwilioSmsError(response.reason)

    return loads(response.read()).get('sid')



from django.core.management.base import BaseCommand
from dbmail.models import MailTemplate


class Command(BaseCommand):
    def handle(self, *args, **options):
        for obj in MailTemplate.objects.all():
            MailTemplate.clean_cache()
            MailTemplate.get_template(obj.slug)



# -*- encoding: utf-8 -*-

from httplib import HTTPSConnection
from urllib import urlencode
from urlparse import urlparse
from json import dumps

from django.conf import settings

from dbmail import get_version
from dbmail.providers.prowl.push import from_unicode


class SlackError(Exception):
    pass


def send(channel, message, **kwargs):
    """
    Site: https://slack.com
    API: https://api.slack.com
    Desc: real-time messaging
    """
    headers = {
        "Content-type": "application/x-www-form-urlencoded",
        "User-Agent": "DBMail/%s" % get_version(),
    }

    username = from_unicode(kwargs.pop("username", settings.SLACK_USERNAME))
    hook_url = from_unicode(kwargs.pop("hook_url", settings.SLACK_HOOCK_URL))
    channel = from_unicode(channel or settings.SLACK_CHANNEL)
    emoji = from_unicode(kwargs.pop("emoji", ""))
    message = from_unicode(message)

    data = {
        "channel": channel,
        "username": username,
        "text": message,
        "icon_emoji": emoji,
    }

    _data = kwargs.pop('data', None)
    if _data is not None:
        data.update(_data)

    up = urlparse(hook_url)
    http = HTTPSConnection(up.netloc)
    http.request(
        "POST", up.path,
        headers=headers,
        body=urlencode({"payload": dumps(data)}))
    response = http.getresponse()

    if response.status != 200:
        raise SlackError(response.reason)

    body = response.read()
    if body != "ok":
        raise SlackError(repr(body))
    return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dbmail', '0002_auto_20150321_1539'),
    ]

    operations = [
        migrations.AddField(
            model_name='maillog',
            name='backend',
            field=models.CharField(default=b'mail', editable=False, choices=[(b'tts', b'dbmail.backends.tts'), (b'mail', b'dbmail.backends.mail'), (b'sms', b'dbmail.backends.sms')], max_length=25, verbose_name='Backend', db_index=True),
            preserve_default=True,
        ),
    ]



# -*- encoding: utf-8 -*-

from dbmail.defaults import BOT_PROVIDER, DEFAULT_BOT_FROM
from dbmail.backends.sms import (
    Sender as SenderBase,
    SenderDebug as SenderDebugBase
)
from dbmail.utils import clean_html
from dbmail import import_module


class Sender(SenderBase):
    provider = BOT_PROVIDER
    default_from = DEFAULT_BOT_FROM

    def _get_recipient_list(self, recipient):
        return self._email_to_list(recipient)

    def _send(self):
        module = import_module(self._provider)
        message = clean_html(self._message)
        for _id in self._recipient_list:
            module.send(_id, message, _from=self._from_email, **self._kwargs)


class SenderDebug(SenderDebugBase):
    pass



# -*- encoding: utf-8 -*-

from httplib import HTTPSConnection
from json import dumps, loads
from urlparse import urlparse

from django.conf import settings


class GCMError(Exception):
    pass


def send(user, message, **kwargs):
    """
    Site: https://developers.google.com
    API: https://developers.google.com/cloud-messaging/
    Desc: Android notifications
    """

    headers = {
        "Content-type": "application/json",
        "Authorization": "key=" + kwargs.pop("gcm_key", settings.GCM_KEY)
    }

    hook_url = 'https://android.googleapis.com/gcm/send'

    data = {
        "registration_ids": [user],
        "data": {
            "title": kwargs.pop("event"),
            'message': message,
        }
    }
    data['data'].update(kwargs)

    up = urlparse(hook_url)
    http = HTTPSConnection(up.netloc)
    http.request(
        "POST", up.path,
        headers=headers,
        body=dumps(data))
    response = http.getresponse()

    if response.status != 200:
        raise GCMError(response.reason)

    body = response.read()
    if loads(body).get("failure") > 0:
        raise GCMError(repr(body))
    return True



from dbmail.providers.microsoft.base import MPNSBase, ElementTree


class MPNSTile(MPNSBase):
    NOTIFICATION_CLS = 1
    TARGET = 'token'

    @staticmethod
    def clear(parent, element, payload_param, payload):
        if payload_param in payload:
            el = ElementTree.SubElement(parent, element)
            if payload[payload_param] is None:
                el.attrib['Action'] = 'Clear'
            else:
                el.text = payload[payload_param]
            return el

    def payload(self, payload):
        root = ElementTree.Element("{WPNotification}Notification")
        tile = ElementTree.SubElement(root, '{WPNotification}Tile')
        self.attr(tile, 'id', payload)
        self.attr(tile, 'template', payload)
        self.sub(
            tile, '{WPNotification}BackgroundImage',
            'background_image', payload)
        self.clear(tile, '{WPNotification}Count', 'count', payload)
        self.clear(tile, '{WPNotification}Title', 'title', payload)
        self.clear(
            tile, '{WPNotification}BackBackgroundImage',
            'back_background_image', payload)
        self.clear(tile, '{WPNotification}BackTitle', 'back_title', payload)
        self.clear(
            tile, '{WPNotification}BackContent', 'back_content', payload)
        return self.serialize_tree(ElementTree.ElementTree(root))


def send(uri, message, **kwargs):
    kwargs['title'] = kwargs.pop("event", 'App')
    kwargs['text1'] = message
    return MPNSTile().send(uri, kwargs)



# -*- encoding: utf-8 -*-

import os

from django.utils.translation import ugettext_lazy as _
from django.conf import settings


def get_settings(key, default):
    return getattr(settings, key, default)


PRIORITY_STEPS = get_settings('DB_MAILER_PRIORITY_STEPS', (
    (0, _("High")),
    (3, _("Medium")),
    (6, _("Low")),
    (9, _("Deferred")),
))
CELERY_QUEUE = get_settings('DB_MAILER_CELERY_QUEUE', 'default')
PUSH_QUEUE = get_settings('DB_MAILER_PUSH_QUEUE', CELERY_QUEUE)
SMS_QUEUE = get_settings('DB_MAILER_SMS_QUEUE', CELERY_QUEUE)
TTS_QUEUE = get_settings('DB_MAILER_TTS_QUEUE', CELERY_QUEUE)
BOT_QUEUE = get_settings('DB_MAILER_BOT_QUEUE', CELERY_QUEUE)
SIGNALS_QUEUE = get_settings('DB_MAILER_SIGNALS_QUEUE', CELERY_QUEUE)
SIGNALS_MAIL_QUEUE = get_settings('DB_MAILER_SIGNALS_MAIL_QUEUE', CELERY_QUEUE)
SUBSCRIPTION_QUEUE = get_settings('DB_MAILER_SUBSCRIPTION_QUEUE', CELERY_QUEUE)
TRACKING_QUEUE = get_settings('DB_MAILER_TRACKING_QUEUE', CELERY_QUEUE)
ENABLE_CELERY = get_settings('DB_MAILER_ENABLE_CELERY', True)
SHOW_CONTEXT = get_settings('DB_MAILER_SHOW_CONTEXT', False)
READ_ONLY_ENABLED = get_settings('DB_MAILER_READ_ONLY_ENABLED', True)
UPLOAD_TO = get_settings('DB_MAILER_UPLOAD_TO', 'mail_files')
DEFAULT_CATEGORY = get_settings('DB_MAILER_DEFAULT_CATEGORY', None)
DEFAULT_FROM_EMAIL = get_settings('DB_MAILER_DEFAULT_FROM_EMAIL', None)
DEFAULT_SMS_FROM = get_settings('DB_MAILER_DEFAULT_SMS_FROM', None)
DEFAULT_PUSH_FROM = get_settings('DB_MAILER_DEFAULT_PUSH_FROM', None)
DEFAULT_BOT_FROM = get_settings('DB_MAILER_DEFAULT_BOT_FROM', None)
DEFAULT_PRIORITY = get_settings('DB_MAILER_DEFAULT_PRIORITY', 6)
TEMPLATES_PER_PAGE = get_settings('DB_MAILER_TEMPLATES_PER_PAGE', 20)
SEND_RETRY = get_settings('DB_MAILER_SEND_RETRY', 3)
SEND_RETRY_DELAY = get_settings('DB_MAILER_SEND_RETRY_DELAY', 300)
SEND_RETRY_DELAY_DIRECT = get_settings('DB_MAILER_SEND_RETRY_DELAY_DIRECT', 6)
SEND_MAX_TIME = get_settings('DB_MAILER_SEND_MAX_TIME', 30)
WSGI_AUTO_RELOAD = get_settings('DB_MAILER_WSGI_AUTO_RELOAD', False)
UWSGI_AUTO_RELOAD = get_settings('DB_MAILER_UWSGI_AUTO_RELOAD', False)
ENABLE_LOGGING = get_settings('DB_MAILER_ENABLE_LOGGING', True)
ADD_HEADER = get_settings('DB_MAILER_ADD_HEADER', False)
LOGS_EXPIRE_DAYS = get_settings('DB_MAILER_LOGS_EXPIRE_DAYS', 7)
ALLOWED_MODELS_ON_ADMIN = get_settings('DB_MAILER_ALLOWED_MODELS_ON_ADMIN', [
    'MailFromEmailCredential',
    'MailFromEmail',
    'MailCategory',
    'MailTemplate',
    'MailLog',
    'MailGroup',
    'Signal',
    'ApiKey',
    'MailBcc',
    'MailLogTrack',
    'MailBaseTemplate',
    'MailSubscription',
    'MailLogException',
])
MAIL_SUBSCRIPTION_MODEL = get_settings(
    'DB_MAILER_MAIL_SUBSCRIPTION_MODEL', 'dbmail.models.MailSubscription')
AUTH_USER_MODEL = get_settings('AUTH_USER_MODEL', 'auth.User')
USE_CELERY_FOR_ADMIN_TEST = get_settings(
    'DB_MAILER_USE_CELERY_FOR_ADMIN_TEST', True)
CACHE_TTL = get_settings('DB_MAILER_CACHE_TIMEOUT', None)
ENABLE_USERS = get_settings('DB_MAILER_ENABLE_USERS', False)
SIGNAL_DEFERRED_DISPATCHER = get_settings(
    'DB_MAILER_SIGNAL_DEFERRED_DISPATCHER', 'celery')
SIGNAL_DB_DEFERRED_PURGE = get_settings(
    'DB_MAILER_SIGNAL_DB_DEFERRED_PURGE', True)

TRACK_ENABLE = get_settings('DB_MAILER_TRACK_ENABLE', True)
TRACK_PIXEL = get_settings(
    'DB_MAILER_TRACK_PIXEL',
    [
        'image/gif',
        "\x47\x49\x46\x38\x39\x61\x01\x00\x01\x00\x80\x00"
        "\x00\xff\xff\xff\x00\x00\x00\x21\xf9\x04\x01\x00"
        "\x00\x00\x00\x2c\x00\x00\x00\x00\x01\x00\x01\x00"
        "\x00\x02\x02\x44\x01\x00\x3b"
    ]
)
TRACK_HTML = get_settings(
    'DB_MAILER_TRACK_HTML',
    '<table bgcolor="white"><tr><td><font size="-1" color="black">'
    '<img src="%(url)s" width="16" height="16" alt="" title="" border="0">'
    '</font></td></tr></table></center>')

BACKEND = get_settings('DB_MAILER_BACKEND', {
    'mail': 'dbmail.backends.mail',
    'tts': 'dbmail.backends.tts',
    'sms': 'dbmail.backends.sms',
    'push': 'dbmail.backends.push',
    'bot': 'dbmail.backends.bot',
})
_BACKEND = {v: k for k, v in BACKEND.items()}
BACKENDS_MODEL_CHOICES = get_settings('DB_MAILER_BACKENDS_MODEL_CHOICES', (
    (BACKEND.get('mail'), _('MailBox')),
    (BACKEND.get('push'), _('Push')),
    (BACKEND.get('sms'), _('SMS')),
    (BACKEND.get('tts'), _('TTS')),
    (BACKEND.get('bot'), _('BOT')),
))

SMS_PROVIDER = get_settings(
    'DB_MAILER_SMS_PROVIDER', 'dbmail.providers.nexmo.sms')
TTS_PROVIDER = get_settings(
    'DB_MAILER_TTS_PROVIDER', 'dbmail.providers.nexmo.tts')
PUSH_PROVIDER = get_settings(
    'DB_MAILER_PUSH_PROVIDER', 'dbmail.providers.prowl.push')
BOT_PROVIDER = get_settings(
    'DB_MAILER_BOT_PROVIDER', 'dbmail.providers.telegram.bot')
MAIL_PROVIDER = get_settings('DB_MAILER_MAIL_PROVIDER', None)

SAFARI_PUSH_PATH = get_settings(
    'DB_MAILER_SAFARI_PUSH_PATH', settings.STATIC_ROOT or '.')

MESSAGE_HTML2TEXT = get_settings(
    'DB_MAILER_MESSAGE_HTML2TEXT', 'dbmail.utils')

IGNORE_BROWSE_APP = get_settings(
    'DB_MAILER_IGNORE_BROWSE_APP', [
        'south', 'dbmail', 'sessions', 'admin', 'djcelery',
        'auth', 'reversion', 'contenttypes'
    ])

MODEL_HTMLFIELD = get_settings(
    'DB_MAILER_MODEL_HTMLFIELD', 'dbmail.fields.HTMLField')
MODEL_SUBSCRIPTION_DATA_FIELD = get_settings(
    'DB_MAILER_MODEL_SUBSCRIPTION_DATA_FIELD', 'dbmail.fields.DataTextField')
APNS_PROVIDER_DEFAULT_ACTION = get_settings(
    'DB_MAILER_APNS_PROVIDER_DEFAULT_ACTION', 'Show')

DEBUG = settings.DEBUG and get_settings('DB_MAILER_DEBUG', False)



# -*- encoding: utf-8 -*-

from json import dumps, loads

from django.conf import settings
from pywebpush import WebPusher


class GCMError(Exception):
    pass


def send(reg_id, message, **kwargs):
    """
    Site: https://developers.google.com
    API: https://developers.google.com/web/updates/2016/03/web-push-encryption
    Desc: Web Push notifications for Chrome and FireFox

    Installation:
    pip install 'pywebpush>=0.4.0'
    """

    subscription_info = kwargs.pop('subscription_info')

    payload = {
        "title": kwargs.pop("event"),
        "body": message,
        "url": kwargs.pop("push_url", None)
    }
    payload.update(kwargs)

    wp = WebPusher(subscription_info)
    response = wp.send(
        dumps(payload), gcm_key=settings.GCM_KEY,
        ttl=kwargs.pop("ttl", 60))

    if not response.ok or (
            response.text and loads(response.text).get("failure") > 0):
        raise GCMError(response.text)
    return True



# -*- encoding: utf-8 -*-

import json
import sys
import os

from django.http import HttpResponse, Http404, HttpResponseBadRequest
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from django.shortcuts import get_object_or_404
from django.views.generic import View
from django.core.cache import cache

from dbmail.models import ApiKey
from dbmail import db_sender
from dbmail import defaults

from dbmail import signals


allowed_fields = [
    'api_key', 'slug', 'recipient', 'from_email', 'cc', 'bcc',
    'queue', 'retry_delay', 'max_retries', 'retry', 'language',
    'time_limit', 'send_after', 'backend', 'provider',
]


@csrf_exempt
def send_by_dbmail(request):
    if request.method == 'POST':
        kwargs = dict()
        for f in allowed_fields:
            if request.POST.get(f):
                kwargs[f] = request.POST.get(f)

        backend = defaults.BACKEND.get(kwargs.pop('backend', 'mail'))
        api_key = kwargs.get('api_key')
        if api_key:
            del kwargs['api_key']
            if not cache.get(api_key):
                get_object_or_404(
                    ApiKey, api_key=api_key, is_active=True)
                cache.set(api_key, 1, timeout=defaults.CACHE_TTL)

            args = []
            if request.POST.get('data'):
                args = [json.loads(request.POST['data'])]

            if kwargs.get('slug') and kwargs.get('recipient'):
                if backend is not None:
                    kwargs['backend'] = backend

                db_sender(
                    kwargs.pop('slug'), kwargs.pop('recipient'),
                    *args, **kwargs)
                return HttpResponse('OK')
    raise Http404


def mail_read_tracker(request, encrypted):
    if defaults.TRACK_ENABLE and defaults.ENABLE_LOGGING:
        from dbmail.tasks import mail_track

        req = {k: v for k, v in request.META.items()
               if k.startswith('HTTP_') or k.startswith('REMOTE')}
        if defaults.ENABLE_CELERY is True:
            mail_track.apply_async(
                args=[req, encrypted], queue=defaults.TRACKING_QUEUE,
                retry=1, retry_policy={'max_retries': 3})
        else:
            mail_track(req, encrypted)

    return HttpResponse(
        content=defaults.TRACK_PIXEL[1],
        content_type=defaults.TRACK_PIXEL[0],
    )


class PostCSRFMixin(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(PostCSRFMixin, self).dispatch(request, *args, **kwargs)


class SafariPushPackagesView(PostCSRFMixin):
    def post(self, _, version, site_pid):
        signals.safari_push_package.send(
            self.__class__, instance=self, version=version, site_pid=site_pid)

        pp = os.path.join(defaults.SAFARI_PUSH_PATH, '%s.zip' % site_pid)
        return HttpResponse(open(pp).read(), content_type='application/zip')


class SafariSubscriptionView(PostCSRFMixin):
    http_method_names = ['post', 'delete']

    def post(self, _, **kwargs):
        signals.safari_subscribe.send(self.__class__, instance=self, **kwargs)
        return HttpResponse()

    def delete(self, _, **kwargs):
        signals.safari_unsubscribe.send(
            self.__class__, instance=self, **kwargs)
        return HttpResponse()


class SafariLogView(PostCSRFMixin):
    def post(self, request, version):
        err = json.loads(request.body)
        signals.safari_error_log.send(self.__class__, instance=self, err=err)
        return HttpResponse()


class PushSubscriptionView(View):
    http_method_names = ['post', 'delete']

    def _process(self, request, signal, **kwargs):
        try:
            kwargs.update(json.loads(request.body))
            signal.send(self.__class__, instance=self, **kwargs)
            return HttpResponse()
        except ValueError:
            return HttpResponseBadRequest()

    def post(self, request, **kwargs):
        return self._process(request, signals.push_subscribe, **kwargs)

    def delete(self, request, **kwargs):
        return self._process(request, signals.push_unsubscribe, **kwargs)



from io import BytesIO
from xml.etree import ElementTree

from django.conf import settings
from requests import post


class WPError(Exception):
    pass


class MPNSBase(object):
    NOTIFICATION_CLS = None
    TARGET = None

    def __init__(self):
        self.headers = {
            'Content-Type': 'text/xml',
            'Accept': 'application/*',
            'X-NotificationClass': self.NOTIFICATION_CLS,
            'X-WindowsPhone-Target': self.TARGET,
        }
        ElementTree.register_namespace('wp', 'WPNotification')

    @staticmethod
    def serialize_tree(tree):
        buf = BytesIO()
        tree.write(buf, encoding='utf-8')
        contents = (
            '<?xml version="1.0" encoding="utf-8"?>' +
            buf.getvalue())
        buf.close()
        return contents

    @staticmethod
    def attr(element, payload_param, payload):
        if payload_param in payload:
            element.attrib['attribute'] = payload[payload_param]

    @staticmethod
    def sub(parent, element, payload_param, payload):
        if payload_param in payload:
            el = ElementTree.SubElement(parent, element)
            el.text = payload[payload_param]
            return el

    def payload(self, payload):
        raise NotImplementedError

    def send(self, uri, payload, msg_id=None, callback_uri=None):
        if msg_id is not None:
            self.headers['X-MessageID'] = msg_id

        if callback_uri is not None:
            self.headers['X-CallbackURI'] = callback_uri

        res = post(
            uri, data=self.payload(payload), headers=self.headers,
            cert=settings.WP_CERT_FILE)
        status = res.headers.get('x-notificationstatus')
        if res.status_code == 200 and status != 'QueueFull':
            return True
        raise WPError(res.reason)



# -*- encoding: utf-8 -*-

from modeltranslation.translator import translator, TranslationOptions
from dbmail.models import MailTemplate, MailBaseTemplate


class MailTemplateTranslationOptions(TranslationOptions):
    fields = ('subject', 'message',)


class MailBaseTemplateTranslationOptions(TranslationOptions):
    fields = ('message',)


translator.register(MailTemplate, MailTemplateTranslationOptions)
translator.register(MailBaseTemplate, MailBaseTemplateTranslationOptions)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('dbmail', '0005_auto_20150506_2201'),
    ]

    operations = [
        migrations.CreateModel(
            name='MailSubscription',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('backend', models.CharField(max_length=50, choices=[(b'dbmail.backends.mail', 'MailBox'), (b'dbmail.backends.push', 'Push'), (b'dbmail.backends.sms', 'SMS'), (b'dbmail.backends.tts', 'TTS')])),
                ('start_hour', models.CharField(default=b'00:00', max_length=5)),
                ('end_hour', models.CharField(default=b'23:59', max_length=5)),
                ('is_enabled', models.BooleanField(default=True, db_index=True)),
                ('is_checked', models.BooleanField(default=False, db_index=True)),
                ('defer_at_allowed_hours', models.BooleanField(default=False)),
                ('address', models.CharField(max_length=60)),
                ('user', models.ForeignKey(verbose_name='User', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'verbose_name': 'Mail Subscription',
            },
        ),
    ]



# -*- encoding: utf-8 -*-

from httplib import HTTPSConnection
from urllib import urlencode
from json import loads

from django.conf import settings

from dbmail import get_version
from dbmail.providers.prowl.push import from_unicode


class PushOverError(Exception):
    pass


def send(user, message, **kwargs):
    """
    Site: https://pushover.net/
    API: https://pushover.net/api
    Desc: real-time notifications
    """
    headers = {
        "Content-type": "application/x-www-form-urlencoded",
        "User-Agent": "DBMail/%s" % get_version(),
    }

    title = from_unicode(kwargs.pop("title", settings.PUSHOVER_APP))
    message = from_unicode(message)

    data = {
        "token": settings.PUSHOVER_TOKEN,
        "user": user,
        "message": message,
        "title": title,
        "priority": kwargs.pop("priority", 0)
    }

    _data = kwargs.pop('data', None)
    if _data is not None:
        data.update(_data)

    http = HTTPSConnection(kwargs.pop("api_url", "api.pushover.net"))
    http.request(
        "POST", "/1/messages.json",
        headers=headers,
        body=urlencode(data))
    response = http.getresponse()

    if response.status != 200:
        raise PushOverError(response.reason)

    body = loads(response.read())
    if body.get('status') != 1:
        raise PushOverError(repr(body))
    return True



# -*- encoding: utf-8 -*-

from dbmail.backends.sms import SenderDebug as SenderDebugBase
from dbmail.backends.sms import Sender as SenderBase
from dbmail.defaults import TTS_PROVIDER


class Sender(SenderBase):
    provider = TTS_PROVIDER


class SenderDebug(SenderDebugBase):
    provider = TTS_PROVIDER



# -*- coding: utf-8 -*-

from django.test import TestCase

from dbmail.models import (
    MailTemplate, MailBcc, MailFromEmail, MailFile, MailFromEmailCredential)


class TemplateTestCase(TestCase):
    def __create_template(self):
        return MailTemplate.objects.create(
            name="Site welcome template",
            subject="Welcome",
            message="Welcome to our site. We are glad to see you.",
            slug="welcome",
            is_html=False,
            id=1,
        )

    def __retrieve_named_template_and_check_num_queries(self, num):
        with self.assertNumQueries(num):
            template = MailTemplate.get_template("welcome")
            self.assertEqual(template.pk, 1)
            return template

    ###########################################################################
    def test_retrieve_named_template(self):
        """ create template and check queries """
        self.__create_template()
        self.__retrieve_named_template_and_check_num_queries(3)

    def test_retrieve_named_template_cached(self):
        """ check num of queries for cached template """
        self.test_retrieve_named_template()
        self.__retrieve_named_template_and_check_num_queries(0)

    def test_retrieve_named_template_with_cache_invalidation(self):
        """ invalidate cached template """
        self.test_retrieve_named_template_cached()

        template = MailTemplate.get_template("welcome")
        self.assertEquals(template.subject, "Welcome")

        template.subject = "Hello"
        template.save()

        self.__retrieve_named_template_and_check_num_queries(3)
        self.assertEquals(template.subject, "Hello")

    def test_retrieve_named_template_with_cache_invalidation_cache(self):
        """ check num of queries for cached template after invalidation """
        self.test_retrieve_named_template_with_cache_invalidation()
        template = self.__retrieve_named_template_and_check_num_queries(0)
        self.assertEquals(template.subject, "Hello")

    ###########################################################################
    def test_retrieve_bcc(self):
        """ create template and check bcc """
        self.__create_template()

        template = self.__retrieve_named_template_and_check_num_queries(3)
        self.assertEquals(template.bcc_list, [])

    def test_retrieve_bcc_cached(self):
        """ create template and bcc cache """
        self.test_retrieve_bcc()
        self.__retrieve_named_template_and_check_num_queries(0)

    def test_retrieve_bcc_cached_invalidation(self):
        """ invalidate cached bcc and add new bcc email """
        self.test_retrieve_bcc_cached()

        bcc = MailBcc.objects.create(email="root@local.host")
        template = MailTemplate.get_template("welcome")
        template.bcc_email.add(bcc)
        template.save()

        with self.assertNumQueries(3):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(template.bcc_list, ["root@local.host"])

    def test_retrieve_bcc_cached_invalidation_cache(self):
        """ check cached bcc with email """
        self.test_retrieve_bcc_cached_invalidation()

        with self.assertNumQueries(0):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(template.bcc_list, ["root@local.host"])

    def test_retrieve_bcc_delete(self):
        """ invalidate cached template when object was removed """
        self.test_retrieve_bcc_cached_invalidation_cache()

        for bcc in MailBcc.objects.all():
            bcc.delete()

        with self.assertNumQueries(3):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(template.bcc_list, [])

        self.__retrieve_named_template_and_check_num_queries(0)

    ###########################################################################
    def test_retrieve_from(self):
        """ create template and check default email from """
        self.__create_template()

        template = self.__retrieve_named_template_and_check_num_queries(3)
        self.assertEquals(template.from_email, None)

    def test_retrieve_from_cached(self):
        """ create template and check email from cache """
        self.test_retrieve_from()
        self.__retrieve_named_template_and_check_num_queries(0)

    def test_retrieve_from_cached_invalidation(self):
        """ invalidate cached from and add new email from """
        self.test_retrieve_from_cached()

        mail_from = MailFromEmail.objects.create(
            name="root", email="r@loc.hs", pk=1)
        template = MailTemplate.get_template("welcome")
        template.from_email = mail_from
        template.save()

        with self.assertNumQueries(3):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(
                template.from_email.get_mail_from, "root <r@loc.hs>")

    def test_retrieve_from_cached_invalidation_cache(self):
        """ check cached from with email """
        self.test_retrieve_from_cached_invalidation()

        with self.assertNumQueries(0):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(
                template.from_email.get_mail_from, "root <r@loc.hs>")

    def test_retrieve_from_delete(self):
        """ invalidate cached template when object was removed """
        self.test_retrieve_from_cached_invalidation_cache()

        mail_from = MailFromEmail.objects.get(pk=1)
        mail_from.delete()

        with self.assertNumQueries(3):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(template.from_email, None)

        self.__retrieve_named_template_and_check_num_queries(0)

    ###########################################################################
    def test_retrieve_file(self):
        self.__create_template()

        template = self.__retrieve_named_template_and_check_num_queries(3)
        self.assertEquals(template.files_list, [])

    def test_retrieve_file_cached(self):
        self.test_retrieve_file()
        self.__retrieve_named_template_and_check_num_queries(0)

    def test_retrieve_file_cached_invalidation(self):
        self.test_retrieve_file_cached()

        attachment = MailFile.objects.create(
            template_id=1, name="report.xls", filename="", pk=1)
        attachment.save()

        with self.assertNumQueries(3):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(template.files_list, [attachment])

    def test_retrieve_file_cached_invalidation_cache(self):
        self.test_retrieve_file_cached_invalidation()

        with self.assertNumQueries(0):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(len(template.files_list), 1)

    def test_retrieve_file_delete(self):
        self.test_retrieve_file_cached_invalidation_cache()

        attachment = MailFile.objects.get(pk=1)
        attachment.delete()

        with self.assertNumQueries(3):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(template.files_list, [])

        self.__retrieve_named_template_and_check_num_queries(0)

    ###########################################################################
    def test_retrieve_auth(self):
        self.__create_template()

        template = self.__retrieve_named_template_and_check_num_queries(3)
        self.assertEquals(template.auth_credentials, None)

    def test_retrieve_auth_cached(self):
        self.test_retrieve_auth()
        self.__retrieve_named_template_and_check_num_queries(0)

    def test_retrieve_auth_cached_invalidation(self):
        self.test_retrieve_auth_cached()

        credentials = MailFromEmailCredential.objects.create(
            host="localhost", port=25, id=1)
        credentials.save()

        mail_from = MailFromEmail.objects.create(
            name="root", email="r@loc.hs", credential=credentials)

        template = MailTemplate.get_template("welcome")
        template.from_email = mail_from
        template.save()

        with self.assertNumQueries(4):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(len(template.auth_credentials), 6)
            self.assertEquals(template.auth_credentials['port'], 25)

    def test_retrieve_auth_cached_invalidation_cache(self):
        self.test_retrieve_auth_cached_invalidation()

        with self.assertNumQueries(0):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(len(template.auth_credentials), 6)
            self.assertEquals(template.auth_credentials['port'], 25)

    def test_retrieve_auth_cached_invalidation_parent(self):
        self.test_retrieve_auth_cached_invalidation_cache()

        credentials = MailFromEmailCredential.objects.get(pk=1)
        credentials.port = 587
        credentials.save()

        with self.assertNumQueries(4):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(len(template.auth_credentials), 6)
            self.assertEquals(template.auth_credentials['port'], 587)

    def test_retrieve_auth_cached_invalidation_parent_cached(self):
        self.test_retrieve_auth_cached_invalidation_parent()

        with self.assertNumQueries(0):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(len(template.auth_credentials), 6)
            self.assertEquals(template.auth_credentials['port'], 587)

    def test_retrieve_auth_cached_delete(self):
        self.test_retrieve_auth_cached_invalidation_parent_cached()

        credentials = MailFromEmailCredential.objects.get(pk=1)
        credentials.delete()

        with self.assertNumQueries(3):
            template = MailTemplate.get_template("welcome")
            self.assertEquals(template.auth_credentials, None)
            self.assertEquals(template.from_email, None)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dbmail', '0012_auto_20160915_1340'),
    ]

    operations = [
        migrations.AddField(
            model_name='maillogexception',
            name='ignore',
            field=models.BooleanField(default=False, verbose_name='Ignore'),
        ),
    ]



import json

import requests

from django.conf import settings

from dbmail.providers.prowl.push import from_unicode


class SMSBlissError(Exception):
    pass


def send(to, message, sms_from=None, **kwargs):
    sender = SMSBlissProvider(sms_from=sms_from)
    return sender.send_message(to, message, **kwargs)


class SMSBlissProvider(object):
    login = settings.SMSBLISS_LOGIN
    password = settings.SMSBLISS_PASSWORD
    url = settings.SMSBLISS_API_URL

    def __init__(self, **kwargs):
        self._from_msg = kwargs.pop('sms_from', settings.SMSBLISS_FROM)

    def send_message(self, sms_to, message, **kwargs):
        kwargs.update({
            'messages': [{
                'clientId': 1,
                'phone': sms_to,
                'text': from_unicode(message),
                'sender': self._from_msg
            }],
            'showBillingDetails': True
        })
        return self._send_request(kwargs)

    def _send_request(self, data_to_send):
        data = self.add_credentials(data_to_send)
        response = requests.post(self.url, data=json.dumps(data))

        response_data = response.json()

        is_success = self.is_success_response(response, response_data)

        if not is_success:
            raise SMSBlissError(response_data)

        return response_data

    def add_credentials(self, data_to_send):
        data_to_send.update({
            'login': self.login,
            'password': self.password,
        })
        return data_to_send

    @staticmethod
    def is_success_response(response, response_data):
        if response.status_code != 200:
            return False

        response_messages = response_data.get('messages', [])

        for message in response_messages:
            if isinstance(message, dict):
                if message.get('status') not in ['accepted', 'delivered']:
                    return False
        return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):
    dependencies = [
        ('dbmail', '0004_auto_20150321_2214'),
    ]

    operations = [
        migrations.CreateModel(
            name='MailBaseTemplate',
            fields=[
                ('id', models.AutoField(
                    verbose_name='ID', serialize=False,
                    auto_created=True, primary_key=True)),
                ('name', models.CharField(
                    unique=True, max_length=100, verbose_name='Name')),
                ('message', models.TextField(
                    help_text='Basic template for mail messages. '
                              '{{content}} tag for msg.',
                    verbose_name='Body')),
                ('created', models.DateTimeField(
                    auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(
                    auto_now=True, verbose_name='Updated')),
            ],
            options={
                'verbose_name': 'Mail base template',
                'verbose_name_plural': 'Mail base templates',
            },
        ),
        migrations.AddField(
            model_name='mailtemplate',
            name='base',
            field=models.ForeignKey(
                verbose_name='Basic template', blank=True,
                to='dbmail.MailBaseTemplate', null=True),
        ),
    ]



from json import dumps

from django.conf import settings
from hyper import HTTP20Connection
from hyper.tls import init_context

from dbmail import defaults
from dbmail.providers.apple.errors import APNsError


def send(token_hex, message, **kwargs):
    """
    Site: https://apple.com
    API: https://developer.apple.com
    Desc: iOS notifications

    Installation and usage:
    pip install hyper
    """

    priority = kwargs.pop('priority', 10)
    topic = kwargs.pop('topic', None)

    alert = {
        "title": kwargs.pop("event"),
        "body": message,
        "action": kwargs.pop(
            'apns_action', defaults.APNS_PROVIDER_DEFAULT_ACTION)
    }

    data = {
        "aps": {
            'alert': alert,
            'content-available': kwargs.pop('content_available', 0) and 1
        }
    }
    data['aps'].update(kwargs)
    payload = dumps(data, separators=(',', ':'))

    headers = {
        'apns-priority': priority
    }
    if topic is not None:
        headers['apns-topic'] = topic

    ssl_context = init_context()
    ssl_context.load_cert_chain(settings.APNS_CERT_FILE)
    connection = HTTP20Connection(
        settings.APNS_GW_HOST, settings.APNS_GW_PORT, ssl_context=ssl_context)

    stream_id = connection.request(
        'POST', '/3/device/{}'.format(token_hex), payload, headers)
    response = connection.get_response(stream_id)
    if response.status != 200:
        raise APNsError(response.read())
    return True



from dbmail.providers.microsoft.base import MPNSBase


class MPNSRaw(MPNSBase):
    NOTIFICATION_CLS = 3
    TARGET = 'raw'

    def payload(self, payload):
        return payload


def send(uri, *_, **kwargs):
    return MPNSRaw().send(uri, kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dbmail', '0008_auto_20151007_1918'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mailsubscription',
            name='address',
            field=models.CharField(help_text='Must be phone number/email/token', max_length=60, verbose_name='Address', db_index=True),
        ),
    ]



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'MailSubscription.title'
        db.alter_column('dbmail_mailsubscription', 'title', self.gf('django.db.models.fields.CharField')(max_length=350, null=True))

        # Changing field 'MailSubscription.address'
        db.alter_column('dbmail_mailsubscription', 'address', self.gf('django.db.models.fields.CharField')(max_length=350))

    def backwards(self, orm):

        # Changing field 'MailSubscription.title'
        db.alter_column('dbmail_mailsubscription', 'title', self.gf('django.db.models.fields.CharField')(max_length=100, null=True))

        # Changing field 'MailSubscription.address'
        db.alter_column('dbmail_mailsubscription', 'address', self.gf('django.db.models.fields.CharField')(max_length=60))

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'dbmail.apikey': {
            'Meta': {'object_name': 'ApiKey'},
            'api_key': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '32'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'dbmail.mailbasetemplate': {
            'Meta': {'object_name': 'MailBaseTemplate'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'dbmail.mailbcc': {
            'Meta': {'object_name': 'MailBcc'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'unique': 'True', 'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'dbmail.mailcategory': {
            'Meta': {'object_name': 'MailCategory'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '25'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'dbmail.mailfile': {
            'Meta': {'object_name': 'MailFile'},
            'filename': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'template': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'files'", 'to': "orm['dbmail.MailTemplate']"})
        },
        'dbmail.mailfromemail': {
            'Meta': {'object_name': 'MailFromEmail'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'credential': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['dbmail.MailFromEmailCredential']", 'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'dbmail.mailfromemailcredential': {
            'Meta': {'object_name': 'MailFromEmailCredential'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'fail_silently': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'host': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'port': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'use_tls': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'})
        },
        'dbmail.mailgroup': {
            'Meta': {'object_name': 'MailGroup'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'dbmail.mailgroupemail': {
            'Meta': {'unique_together': "(('email', 'group'),)", 'object_name': 'MailGroupEmail'},
            'email': ('django.db.models.fields.CharField', [], {'max_length': '75'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'emails'", 'to': "orm['dbmail.MailGroup']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'dbmail.maillog': {
            'Meta': {'object_name': 'MailLog'},
            'backend': ('django.db.models.fields.CharField', [], {'default': "'mail'", 'max_length': '25', 'db_index': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'error_exception': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['dbmail.MailLogException']", 'null': 'True', 'blank': 'True'}),
            'error_message': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_sent': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'log_id': ('django.db.models.fields.CharField', [], {'max_length': '60', 'db_index': 'True'}),
            'num_of_retries': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'}),
            'provider': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '250', 'null': 'True', 'db_index': 'True', 'blank': 'True'}),
            'template': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['dbmail.MailTemplate']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'dbmail.maillogemail': {
            'Meta': {'object_name': 'MailLogEmail'},
            'email': ('django.db.models.fields.CharField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'log': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['dbmail.MailLog']"}),
            'mail_type': ('django.db.models.fields.CharField', [], {'max_length': '3'})
        },
        'dbmail.maillogexception': {
            'Meta': {'object_name': 'MailLogException'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '150'})
        },
        'dbmail.maillogtrack': {
            'Meta': {'object_name': 'MailLogTrack'},
            'counter': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip': ('django.db.models.fields.GenericIPAddressField', [], {'max_length': '39'}),
            'ip_area_code': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ip_city': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ip_country_code': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ip_country_code3': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ip_country_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ip_dma_code': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ip_latitude': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ip_longitude': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ip_postal_code': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ip_region': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'is_read': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'mail_log': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['dbmail.MailLog']"}),
            'ua': ('django.db.models.fields.CharField', [], {'max_length': '350', 'null': 'True', 'blank': 'True'}),
            'ua_browser': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'ua_browser_version': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'ua_dist': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'ua_dist_version': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'ua_os': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'ua_os_version': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'dbmail.mailsubscription': {
            'Meta': {'object_name': 'MailSubscription'},
            'address': ('django.db.models.fields.CharField', [], {'max_length': '350', 'db_index': 'True'}),
            'backend': ('django.db.models.fields.CharField', [], {'default': "'dbmail.backends.mail'", 'max_length': '50'}),
            'data': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'defer_at_allowed_hours': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'end_hour': ('django.db.models.fields.CharField', [], {'default': "'23:59'", 'max_length': '5'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_checked': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'is_enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'start_hour': ('django.db.models.fields.CharField', [], {'default': "'00:00'", 'max_length': '5'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '350', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'dbmail.mailtemplate': {
            'Meta': {'object_name': 'MailTemplate'},
            'base': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['dbmail.MailBaseTemplate']", 'null': 'True', 'blank': 'True'}),
            'bcc_email': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['dbmail.MailBcc']", 'symmetrical': 'False', 'blank': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['dbmail.MailCategory']", 'null': 'True', 'blank': 'True'}),
            'context_note': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'enable_log': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'from_email': ('django.db.models.fields.related.ForeignKey', [], {'default': 'None', 'to': "orm['dbmail.MailFromEmail']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'interval': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_admin': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_html': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'num_of_retries': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'}),
            'priority': ('django.db.models.fields.SmallIntegerField', [], {'default': '6'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'dbmail.signal': {
            'Meta': {'object_name': 'Signal'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['dbmail.MailGroup']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'interval': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'model': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'receive_once': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'rules': ('django.db.models.fields.TextField', [], {'default': "'{{ instance.email }}'", 'null': 'True', 'blank': 'True'}),
            'signal': ('django.db.models.fields.CharField', [], {'default': "'post_save'", 'max_length': '15'}),
            'template': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['dbmail.MailTemplate']"}),
            'update_model': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'dbmail.signaldeferreddispatch': {
            'Meta': {'object_name': 'SignalDeferredDispatch'},
            'args': ('django.db.models.fields.TextField', [], {}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'done': ('django.db.models.fields.NullBooleanField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'eta': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'kwargs': ('django.db.models.fields.TextField', [], {}),
            'params': ('django.db.models.fields.TextField', [], {})
        },
        'dbmail.signallog': {
            'Meta': {'object_name': 'SignalLog'},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'model_pk': ('django.db.models.fields.BigIntegerField', [], {}),
            'signal': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['dbmail.Signal']"})
        }
    }

    complete_apps = ['dbmail']


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dbmail', '0009_auto_20160311_0918'),
    ]

    operations = [
        migrations.AddField(
            model_name='mailsubscription',
            name='data',
            field=models.TextField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='mailsubscription',
            name='title',
            field=models.CharField(max_length=100, null=True, blank=True),
        ),
    ]



# -*- encoding: utf-8 -*-

import datetime

from django.core.exceptions import ObjectDoesNotExist
from django.template import Template, Context
from django.contrib.sites.models import Site
from django.contrib.auth.models import User
from django.db.models import signals
from django import dispatch

from dbmail.defaults import (
    SIGNALS_QUEUE, SIGNALS_MAIL_QUEUE, SIGNAL_DEFERRED_DISPATCHER,
    ENABLE_USERS, SEND_RETRY, SEND_RETRY_DELAY
)
from dbmail.models import Signal, SignalDeferredDispatch


class SignalReceiver(object):
    def __init__(self, sender, **kwargs):
        self.sender = sender
        self.kwargs = kwargs
        self._kwargs = kwargs.copy()
        self.site = Site.objects.get_current()
        self.instance = kwargs.get('instance')
        self.pk = self.instance and self.instance.pk or None

        self.signal = None
        self.signal_pk = self.kwargs.pop('signal_pk', None)

        self.kwargs['old_instance'] = self.get_old_instance()
        self.kwargs['users'] = self.get_users()
        self.kwargs['date'] = datetime.date.today()
        self.kwargs['date_time'] = datetime.datetime.now()

    def get_signal_list(self):
        if not hasattr(self.sender._meta, 'module_name'):
            return Signal.objects.filter(
                model__model=self.sender._meta.model_name,
                is_active=True
            )
        return Signal.objects.filter(
            model__model=self.sender._meta.module_name,
            is_active=True
        )

    def get_email_list(self):
        if self.signal.group:
            return self.signal.group.slug

        email_list = Template(self.signal.rules).render(Context(self.kwargs))
        self.kwargs.pop('users', None)
        return email_list.strip().replace('\r', '').replace('\n', '')

    def get_interval(self):
        options = dict()
        if self.signal.interval >= 0 and not self.signal_pk:
            options['send_after'] = self.signal.interval
        return options

    @staticmethod
    def get_users():
        if ENABLE_USERS:
            return User.objects.filter(
                is_active=True, is_staff=False, is_superuser=False)
        return []

    def get_old_instance(self):
        try:
            instance = self.kwargs.get('instance')
            if instance and instance.pk:
                return self.sender.objects.get(
                    pk=self.kwargs['instance'].pk)
        except ObjectDoesNotExist:
            pass

    def get_current_instance(self):
        try:
            if self.instance and self.instance.pk and self.signal.update_model:
                obj = self.instance._default_manager.get(pk=self.instance.pk)
                self.kwargs['current_instance'] = obj
        except ObjectDoesNotExist:
            pass

    def send_mail(self):
        from dbmail import send_db_mail

        email_list = self.get_email_list()
        if email_list and not self.signal.is_sent(self.pk):
            for email in email_list.split(","):
                email = email.strip()
                if email:
                    send_db_mail(
                        self.signal.template.slug, email, self.site,
                        self.kwargs, self.instance, queue=SIGNALS_MAIL_QUEUE,
                        **self.get_interval()
                    )
            self.signal.mark_as_sent(self.pk)

    def _dispatch_deferred_task(self):
        self._kwargs['signal_pk'] = self.signal.pk

        if SIGNAL_DEFERRED_DISPATCHER == 'celery':
            from dbmail import tasks

            tasks.deferred_signal.apply_async(
                args=[self.sender], kwargs=self._kwargs,
                default_retry_delay=SEND_RETRY_DELAY,
                max_retries=SEND_RETRY,
                queue=SIGNALS_QUEUE,
                countdown=self.signal.interval
            )
        else:
            SignalDeferredDispatch.add_task(
                args=[self.sender], kwargs=self._kwargs,
                params=dict(
                    default_retry_delay=SEND_RETRY_DELAY,
                    max_retries=SEND_RETRY,
                    queue=SIGNALS_QUEUE
                ), interval=self.signal.interval
            )

    def _run(self):
        if self.signal.interval:
            self._dispatch_deferred_task()
        else:
            self.send_mail()

    def run(self):
        for self.signal in self.get_signal_list():
            self._run()

    def run_deferred(self):
        try:
            self.signal = Signal.objects.get(pk=self.signal_pk, is_active=True)
            self.get_current_instance()
            self.send_mail()
        except ObjectDoesNotExist:
            pass


def signal_receiver(sender, **kwargs):
    from dbmail import celery_supported

    if 'signal' in kwargs:
        kwargs.pop('signal')

    if celery_supported():
        import tasks

        tasks.signal_receiver.apply_async(
            args=[sender], kwargs=kwargs,
            default_retry_delay=SEND_RETRY_DELAY,
            max_retries=SEND_RETRY,
            queue=SIGNALS_QUEUE,
        )
    else:
        SignalReceiver(sender, **kwargs).run()


def initial_signals():
    for signal in Signal.objects.filter(is_active=True):
        def_signal = getattr(signals, signal.signal)
        def_signal.connect(
            signal_receiver, sender=signal.model.model_class(),
            dispatch_uid=signal.model.name
        )


pre_send = dispatch.Signal()
post_send = dispatch.Signal()
post_exception = dispatch.Signal()

safari_push_package = dispatch.Signal()
safari_subscribe = dispatch.Signal()
safari_unsubscribe = dispatch.Signal()
safari_error_log = dispatch.Signal()

push_subscribe = dispatch.Signal()
push_unsubscribe = dispatch.Signal()
